#include "highscoredlgbase.h"
#include "highscoredlg.h"
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include <qgroupbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qheader.h>
#include <qdatetime.h>
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>

#define MAX_ENTRIES 10

/* 
 *  Constructs a HighScoreDlg which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
HighScoreDlg::HighScoreDlg( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : HighscoreDlgBase( parent, name, modal, fl )
{
  mScore=0;
  mConfig=0;
  mMaxNo=0;
  mSave=false;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
HighScoreDlg::~HighScoreDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

void HighScoreDlg::slotResetData()
{
  if (!mConfig) return ;
  if (KMessageBox::Yes==KMessageBox::warningYesNo(this,
                       i18n("Do you really want to clear all highscore data?")))
  {
    ListView1->clear();
    mSave=true;
  }
}

void HighScoreDlg::enterName(KConfig *config,QString levelname,QString savename,int score)
{
  mConfig=config;
  connect(this,SIGNAL(user1Clicked()),this,SLOT(slotResetData()));
  if (!config) return ;
  mLevel=savename;
  TextLabel1_2->setText(levelname);
  ListView1->setSorting(-1);
  ListView1->addColumn( i18n( "Position" ) );
  ListView1->header()->setClickEnabled( FALSE, ListView1->header()->count() - 1 );
  ListView1->addColumn( i18n( "Score" ) );
  ListView1->header()->setClickEnabled( FALSE, ListView1->header()->count() - 1 );
  ListView1->addColumn( i18n( "Name" ) );
  ListView1->header()->setClickEnabled( FALSE, ListView1->header()->count() - 1 );
  ListView1->addColumn( i18n( "Date" ) );
  ListView1->header()->setClickEnabled( FALSE, ListView1->header()->count() - 1 );
  ListView1->setAllColumnsShowFocus( TRUE );
  // build list
  config->setGroup(QString("highscores_%1").arg(mLevel));
  int no=config->readNumEntry("number",0);
  mMaxNo=config->readNumEntry("maxnumber",MAX_ENTRIES);
  QString lastname=config->readEntry("lastname");
  //kdDebug() << "MaxNo="<< mMaxNo << " score=" << score << " no=" << no << endl;
  if (mMaxNo<1) mMaxNo=1;
  QListViewItem *item=0;
  bool canEnter=no<mMaxNo;
  for (int i=0;i<no;i++)
  {
    if (i>=mMaxNo) continue;
    QString name=QString("name%1").arg(i);
    QString date=QString("date%1").arg(i);
    QString rscore=QString("score%1").arg(i);
    int sc=config->readNumEntry(rscore,0);
    QString na=config->readEntry(name,i18n("nobody"));
    QString da=config->readEntry(date,QString("---"));
    item = new QListViewItem( ListView1, item );
    item->setText( 0,QString("%1").arg(i+1)  );
    item->setText( 1,QString("%1").arg(sc)  );
    item->setText( 2, na );
    item->setText( 3, da );
    if (score>sc) canEnter=true;
  }
  if (score && canEnter)
  {
    GroupBox_Name->show();
    LCDNumber_Name->display(score);
    LineEdit_Name->setText(lastname);
    connect(PushButton_Name,SIGNAL(clicked()),this,SLOT(slotEnterName()));
    connect(this,SIGNAL(okClicked()),this,SLOT(slotOkClicked()));
    mScore=score;
  }
  else
  {
    GroupBox_Name->hide();
    mScore=0;
  }
  mSave=false;
}
void HighScoreDlg::saveData()
{
  if (!mConfig) return ;
  if (!mSave) return ; // no need to save
  int no=0;
  QListViewItem *it=ListView1->firstChild();
  mConfig->setGroup(QString("highscores_%1").arg(mLevel));
  while(it)
  {
    if (no>=mMaxNo) break;
    QString text=it->text(1);
    int sc=text.toInt();
    QString na=it->text(2);
    QString da=it->text(3);
    QString name=QString("name%1").arg(no);
    QString date=QString("date%1").arg(no);
    QString rscore=QString("score%1").arg(no);
    it=it->nextSibling();
    no++;
    mConfig->writeEntry(rscore,sc);
    mConfig->writeEntry(name,na);
    mConfig->writeEntry(date,da);
  }
  mConfig->writeEntry("number",no);
  mConfig->writeEntry("maxnumber",mMaxNo);
  mConfig->writeEntry("lastname",mLastName);
  //kdDebug() << "saveData::LastName=" << mLastName << endl;
}

void HighScoreDlg::slotOkClicked()
{
  if (GroupBox_Name->isEnabled()) slotEnterName();
}
void HighScoreDlg::slotEnterName()
{
  QString s=LineEdit_Name->text();
  if (s.isNull()) return ;
  mLastName=s;
  mSave=true;
  //PushButton_Name->setDisabled(true);
  //LineEdit_Name->setDisabled(true);
  GroupBox_Name->setDisabled(true);
  int i=0;

  QListViewItem *after=0;
  QListViewItem *it=ListView1->firstChild();
  bool hasFound=false;
  while(it)
  {
    QString text=it->text(1);
    QString pos=it->text(0);
    int score=text.toInt();
    if (mScore<=score)
    {
      after=it;
    }
    else if (score<mScore && !hasFound)
    {
      hasFound=true;
      i=pos.toInt();
    }
    if (hasFound)
    {
      pos=QString("%1").arg(pos.toInt()+1);
      it->setText(0,pos);
    }
    it=it->nextSibling();
  }

  if (i==0) i=1; // only one item
  QListViewItem * item = new QListViewItem( ListView1, after );
  item->setText( 0,QString("%1").arg(i)  );
  item->setText( 1,QString("%1").arg(mScore)  );
  item->setText( 2, s );

  QTime time=QTime::currentTime();
  QDate date=QDate::currentDate();
  QString da=date.toString() + QString(" ")+time.toString();
  item->setText( 3, da );

}
