/*
                          quintalign.cpp  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprogressdialog.h>

// include files for KDE
#include <kstddirs.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <khelpmenu.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kdebug.h>
#include <kstatusbar.h>


// application specific includes
#include "quintalign.h"
#include "quintalignview.h"
#include "quintaligndoc.h"
#include "techdlg.h"
#include "userlevel.h"


#include <stdlib.h>

#define ACTION(x)   (actionCollection()->action(x))
#define ID_STATUS_MSG               1003
#define ID_STATUS_MOVER              1002
#define ID_STATUS_TIME               1001

#define DEFAULT_LEVEL 2

GameApp::GameApp() : KMainWindow(0)
{
  config=kapp->config();

    KGlobal::dirs()->addResourceType("quintagrafix", KStandardDirs::kde_default("data") + QString::fromLatin1("quintalign/grafix/"));
#ifdef SRCDIR
    kdDebug() << "SRCDIR=" << SRCDIR << endl;
    KGlobal::dirs()->addResourceDir("quintagrafix", SRCDIR + QString::fromLatin1("/../grafix/"));
#endif

    QStringList list = KGlobal::dirs()->findAllResources("quintagrafix", "*/index.desktop", false, true);
    if (global_debug>0)
    {
      kdDebug() << "dircount=insert " << list.count() << endl;
      kdDebug() << "dirt " << list[0] << endl;
      kdDebug() << "dirt " << list[1] << endl;
    }
    
    for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
    {
        QString path = (*it).left((*it).findRev('/') + 1);

        KSimpleConfig cfg(*it);
        cfg.setGroup(QString::fromLatin1("Quintalign Theme"));
        QString name=cfg.readEntry("Name", i18n("unnamed"));
        if (global_debug>0)
          kdDebug() <<"Name="<<name<< " path="<<path << endl;
        mGrafixDir.append(path);
        mGrafixName.append(name);
        if (cfg.readBoolEntry("default",false))
        {
          mCurrentTheme=path;
          mCurrentThemeName=name;
        }
    }

    if (list.isEmpty())
    {
      kdDebug() << "Error: No graphics found...programm will not work...."<<endl;
      return ;
    }

    config->setGroup("Theme");
    QString stheme=config->readEntry("path");
    // Theme exists?
    if (mGrafixDir.end()!=mGrafixDir.find(stheme))
    {
      if (global_debug>0)
        kdDebug() << " Setting stored theme" << endl;
      mCurrentTheme=stheme;
      mCurrentThemeName=config->readEntry("name",mCurrentThemeName);
    }


    mGrafix=mCurrentTheme;
    if (global_debug>0)
      kdDebug() << "Grafixdir=" << mGrafix << endl;

#ifdef ABC
   // localise data file
   QString file;
   file=QString::fromLatin1("quintalign/grafix/background.png");
   mGrafix=kapp->dirs()->findResourceDir("data", file);
   if (!mGrafix.isNull()) mGrafix+=QCString("quintalign/grafix/");
   else
   {
     file=QString::fromLatin1("grafix/background.png");
     QFile f(file);
     if (f.exists()) mGrafix=QCString("grafix/");
     else
     {
      file=QString::fromLatin1("../grafix/background.png");
      QFile f1(file);
      if (f1.exists()) mGrafix=QCString("../grafix/");
      else kdError() << "Cannot find direcory of graphic files"  << endl;
     }
   }
   if (global_debug>3)
     kdDebug() << "Localised datafile=" << mGrafix << endl;
#endif

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initGUI();
  initStatusBar();
  createGUI(QString::null, false);

  initDocument();
  initView();


  
//  setMinimumSize(640,480);
//  setMaximumSize(800,600);
  resize( 710, 570 );

  readOptions();
  prepareLevels();

  // better be last in init
  checkMenus(All);
  slotWallpaper(); // After check menus should set the right wallpaper...

  // To start the event loop first
  QTimer::singleShot(0,this,SLOT(slotIntro()));

}
void GameApp::slotIntro()
{
  doc->startIntro();
}

GameApp::~GameApp()
{
}

void GameApp::checkMenus(int menu)
{
  if (!menu || (menu&CheckFileMenu))
  {
    if (doc->isRunning()) disableAction("game_new");
    else enableAction("game_new");

    if (!doc->isRunning()) disableAction("game_end");
    else enableAction("game_end");
  }
  if (!menu || (menu&CheckViewMenu))
  {
    if (statusBar()->isHidden()) ((KToggleAction*)ACTION("view_statusbar"))->setChecked(false);
    else ((KToggleAction*)ACTION("view_statusbar"))->setChecked(true);
    if (doc->decoration()) ((KToggleAction*)ACTION("view_decorations"))->setChecked(true);
    else ((KToggleAction*)ACTION("view_decorations"))->setChecked(false);
  }
  if (!menu || (menu&CheckEditMenu))
  {
    if (!doc->isRunning() || doc->noRedraw()) disableAction("edit_redraw");
    else enableAction("edit_redraw");
  }

  if (!menu || (menu&CheckOptionsMenu))
  {
    int no=doc->wallpaper();
    ((KSelectAction *)ACTION("settings_wallpaper"))->setCurrentItem(no);

    ((KSelectAction *)ACTION("settings_theme"))->setCurrentItem(mGrafixDir.findIndex(mCurrentTheme));

    if (doc->isRunning()) disableAction("settings_level");
    else enableAction("settings_level");

    int level=mLevelList.findIndex(doc->level());
    ((KSelectAction *)ACTION("settings_level"))->setCurrentItem(level);
  }

}

void GameApp::initGUI()
{
  QStringList list;
  
  QString dots("...");
  (void)KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection(), "game_new");
  ACTION("game_new")->setStatusText(i18n("Start a new game")+dots);
  ACTION("game_new")->setWhatsThis(i18n("Start a new game"));

  (void)new KAction(i18n("&End Game"),"stop", 0, this, SLOT(slotFileEnd()),
                      actionCollection(), "game_end");
  ACTION("game_end")->setStatusText(i18n("End the current game")+dots);
  ACTION("game_end")->setWhatsThis(i18n("End the current game"));

  (void)new KAction(i18n("&Show Highscores..."),"flag", 0, this, SLOT(slotFileHighscores()),
                      actionCollection(), "game_statistics");
  ACTION("game_statistics")->setStatusText(i18n("Show highscores for the current level")+dots);
  ACTION("game_statistics")->setWhatsThis(i18n("Show highscores for the current level"));

  (void)KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection(), "game_exit");
  ACTION("game_exit")->setStatusText(i18n("Quits the program")+dots);
  ACTION("game_exit")->setWhatsThis(i18n("Quits the program"));

  if (global_debug>0)
  {
    (void)new KAction(i18n("&Debug Game"),0, 0, this, SLOT(slotFileDebug()),
                        actionCollection(), "game_debug");
  }

  (void)new KAction(i18n("&Draw new pieces"),0,  this, SLOT(slotRedraw()),
                      actionCollection(), "edit_redraw");
  ACTION("edit_redraw")->setStatusText(i18n("Draw new pieces")+dots);
  ACTION("edit_redraw")->setWhatsThis(i18n("Draw new pieces"));

  (void)new KToggleAction(i18n("View &Statusbar"),0,this,SLOT(slotViewStatusBar()),
                      actionCollection(), "view_statusbar");
  ACTION("view_statusbar")->setStatusText(i18n("Toggle the statusbar")+dots);
  ACTION("view_statusbar")->setWhatsThis(i18n("Toggle the statusbar"));

  (void)new KToggleAction(i18n("View &Decorations"),0,this,SLOT(slotViewDecorations()),
                      actionCollection(), "view_decorations");
  ACTION("view_decorations")->setStatusText(i18n("Toggle the display of the decorations")+dots);
  ACTION("view_decorations")->setWhatsThis(i18n("Toggle the display of the decorations"));

  (void)new KSelectAction(i18n("&Level"),0,this,SLOT(slotLevel()),
                      actionCollection(), "settings_level");
  ACTION("settings_level")->setStatusText(i18n("Choose a level of difficulty")+dots);
  ACTION("settings_level")->setWhatsThis(i18n("Choose a level of difficulty"));

  KGlobal::dirs()->addResourceType("wallpaper", KStandardDirs::kde_default("data") + "ksnake/backgrounds/");
  (void) new KSelectAction(i18n("&Change Background"), 0, this,
                              SLOT(slotWallpaper()),
                              actionCollection(), "settings_wallpaper");
  list.clear();
  wallpaperlist = KGlobal::dirs()->findAllResources("wallpaper", "*.jpg", false, true, list);
  for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) *it = (*it).left((*it).length() - 4);
  list.prepend(i18n("Default"));
  ((KSelectAction *)ACTION("settings_wallpaper"))->setItems(list);
  ACTION("settings_wallpaper")->setStatusText(i18n("Choose a background pattern")+dots);
  ACTION("settings_wallpaper")->setWhatsThis(i18n("Choose a background pattern"));

  (void) new KSelectAction(i18n("&Theme"), 0, this,
                              SLOT(slotTheme()), actionCollection(), "settings_theme");
  ((KSelectAction *)ACTION("settings_theme"))->setItems(mGrafixName);
  ACTION("settings_theme")->setStatusText(i18n("Choose a theme (if installed)")+dots);
  ACTION("settings_theme")->setWhatsThis(i18n("Choose a theme (if installed)"));


  (void)new KAction(i18n("Configure &animations..."),0,  this, SLOT(slotTechnical()),
                      actionCollection(), "settings_technical");
  ACTION("settings_technical")->setStatusText(i18n("Change movement and animation speed")+dots);
  ACTION("settings_technical")->setWhatsThis(i18n("Change movement and animation speed"));

 
  (void)new KAction(i18n("&Userlevel management..."),0,  this, SLOT(slotUserlevel()),
                      actionCollection(), "settings_userlevel");
  ACTION("settings_userlevel")->setStatusText(i18n("Create, modify and delete user defined level")+dots);
  ACTION("settings_userlevel")->setWhatsThis(i18n("Create, modify and delete user defined level"));

 
  actionCollection()->setHighlightingEnabled(true);
  connect(actionCollection(), SIGNAL(actionStatusText(const QString &)), SLOT(slotStatusMsg(const QString &)));
  connect(actionCollection(), SIGNAL(clearStatusText()), SLOT(slotClearStatusMsg()));
}


void GameApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // statusBar()->setInsertOrder(KStatusBar::RightToLeft);
  statusBar()->insertItem(i18n("This leaves space for the mover"),ID_STATUS_MOVER,0,true);
  statusBar()->insertItem(i18n("23:45"),ID_STATUS_TIME,0,true);
  statusBar()->insertItem(i18n("Ready"), ID_STATUS_MSG);

  slotStatusTime();
  slotStatusMover(i18n("(c) Martin Heni   "));
  slotStatusMsg(i18n("Welcome to quintalign"));

  // status bar clock
  statusTimer=new QTimer(this);
  connect(statusTimer,SIGNAL(timeout()),this,SLOT(slotStatusTimer()));
  statusTimer->start(10000,false);
}

// Create and init the game document
void GameApp::initDocument()
{
  // New document
  doc = new GameDoc(this);
  // Start up the game data
  doc->newDocument(config);
  // Connect a few important signals
  connect(doc,SIGNAL(gameOver(int)),this,SLOT(checkMenus(int)));
  connect(doc,SIGNAL(statusMove(const QString &)),this,SLOT(slotStatusMsg(const QString &)));
}

// Create and initilize the game view
void GameApp::initView()
{
  // New canvas view
  view = new GameView(mGrafix,this);
  // REgister it in the document
  doc->setView(view);
  // And set it as out widget
  setCentralWidget(view);
  // No caption needed
  setCaption(0);
}

// Enable the action with the given name
void GameApp::enableAction(const char *s)
{
  if (s)
  {
    KAction *act=actionCollection()->action(s);
    if (act) act->setEnabled(true);
  }
    
}

// Disable the action with the given name
void GameApp::disableAction(const char *s)
{
  if (s)
  {
    KAction *act=actionCollection()->action(s);
    if (act) act->setEnabled(false);
  }
}

// Return a pointer to our document
GameDoc *GameApp::getDocument() const
{
  return doc;
}

void GameApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  doc->WriteConfig(config);
}


void GameApp::readOptions()
{
  config->setGroup("General Options");
  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  ((KToggleAction *)ACTION("view_statusbar"))->setChecked(bViewStatusbar);
  if(!bViewStatusbar) statusBar()->hide();

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    if (global_debug>3)
      kdDebug() << "readOptions size="<< size.width()<<" "<<size.height()<<endl;
    resize(size);
  }
  doc->ReadConfig(config);
}

bool GameApp::queryClose()
{
  return true;
}

bool GameApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void GameApp::slotFileDebug()
{
  doc->Debug();
}

void GameApp::slotFileHighscores()
{
  doc->showHighscores();
}


void GameApp::slotFileNew()
{
  NewGame();
  checkMenus(CheckFileMenu|CheckOptionsMenu|CheckEditMenu);
}

void GameApp::slotFileEnd()
{
  EndGame();
  checkMenus(CheckFileMenu|CheckOptionsMenu|CheckEditMenu);
}


void GameApp::slotFileQuit()
{
  // Save the interal properties
  saveOptions();

  // Close view and mainwindow
  view->close();
  close();
}

void GameApp::slotRedraw()
{
  doc->redrawNewPieces();
}

void GameApp::slotTheme()
{
  int current=((KSelectAction *)ACTION("settings_theme"))->currentItem();
  QString path=mGrafixDir[current];
  QString name=mGrafixName[current];
  if (global_debug>2)
    kdDebug() << "slotTheme - theme " << current << "name=" << name << " path=" << path << endl;
  mCurrentTheme=path;
  mCurrentThemeName=name;
  config->setGroup("Theme");
  config->writeEntry("path",mCurrentTheme);
  config->writeEntry("name",mCurrentThemeName);
  config->sync();

  mGrafix=mCurrentTheme;
  if (global_debug>0)
    kdDebug() << "Grafixdir=" << mGrafix << endl;
  doc->changeTheme(mGrafix);
}
void GameApp::slotWallpaper()
{
  QPixmap background;
  int no=((KSelectAction *)ACTION("settings_wallpaper"))->currentItem();
  if (global_debug>0)
  {
    if (no>0) kdDebug() << "Setting background no=" << no << " name=" <<locate("wallpaper", wallpaperlist[no-1]) <<  endl;
    else kdDebug() << " Default background chosen" << endl;
  }
  //if (no==0) background = QPixmap();
  //else background = QPixmap(locate("wallpaper", wallpaperlist[no-1]));
  bool result=true;
  if (no>0) result=background.load(locate("wallpaper", wallpaperlist[no-1]),"JPEG");
  doc->setBackground(background);
  doc->setWallpaper(no);
}
void GameApp::slotTechnical()
{
  TechDlg *dlg=new TechDlg(this,"techdlg");
  doc->setTechDlg(dlg);
  if (dlg->exec()==QDialog::Accepted)
  {
    doc->getTechDlg(dlg);
  }
  delete dlg;
}

void GameApp::prepareLevels()
{
  QString levelname;
  // int level=0;
  doc->getLevels(mLevelList,0);
  ((KSelectAction *)ACTION("settings_level"))->setItems(mLevelList);
  checkMenus(CheckOptionsMenu);
}

void GameApp::slotUserlevel()
{
  UserlevelDlg *dlg=new UserlevelDlg(this,"userleveldlg");

  QStringList list;
  QString levelname;
  int level=0;
  doc->getLevels(list,1);
  dlg->setItems(list);
  dlg->setConfig(config);
  dlg->exec();
  dlg->writeItems(config);
  delete dlg;
  doc->readLevels(config);
  prepareLevels();
  level=mLevelList.findIndex(doc->level());
  // current level deleted
  if (level<0) 
  {
    doc->setLevel(mLevelList[DEFAULT_LEVEL]);
    checkMenus(CheckOptionsMenu);
  }
}
void GameApp::slotLevel()
{
  QString s=((KSelectAction *)ACTION("settings_level"))->currentText();
  doc->setLevel(s);
}

//turn Statusbar on or off
void GameApp::slotViewStatusBar()
{
  if (statusBar()->isVisible()) 
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }
}
//turn decorations on or off
void GameApp::slotViewDecorations()
{
  doc->setDecoration(!doc->decoration());
}

// Show Ready in the statusbar
void GameApp::slotClearStatusMsg()
{
  slotStatusMsg(i18n("Ready"));
}

// change status message permanently
void GameApp::slotStatusMsg(const QString &text)
{
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

// change status mover permanently
void GameApp::slotStatusMover(const QString &text)
{
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MOVER);
}

// change status time permanently
void GameApp::slotStatusTime()
{
  QString s;
  QTime now;

  now=QTime::currentTime();
  s=now.toString();
  s.truncate(5);

  statusBar()->clear();
  statusBar()->changeItem(s, ID_STATUS_TIME);
}


// change status message of whole statusbar temporary (text, msec)
void GameApp::slotStatusHelpMsg(const QString &text)
{
  statusBar()->message(text, 2000);
}


/** Triggers the status timer */
void GameApp::slotStatusTimer(void)
{
  slotStatusTime();
}

/** Set the names in the mover field */
void GameApp::slotStatusNames(){

  QString msg;
  msg=i18n("No game running");
  // TODO
  slotStatusMover(msg);
}

void GameApp::NewGame()
{
  doc->newGame();
}
void GameApp::EndGame()
{
  doc->endGame();
}

void GameApp::slotHelpAbout()
{
}


void GameApp::SetGrafix(QString s)
{
  mGrafix=s;
}

