/***************************************************************************
                          quintalign.h  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUINTALIGN_H
#define QUINTALIGN_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt
#include <qstrlist.h>

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>

// Window update codes
#define UPDATE_STATUS  1

extern int global_debug;
// forward declaration of the Game classes
class GameDoc;
class GameView;

/**
  * The base class for Game application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of GameView creates your center view, which is connected
  * to the window's Doc object.
  * GameApp reimplements the methods that KMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  * @see KAccel
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
  * @version KDevelop version 0.4 code generation
  */
class GameApp : public KMainWindow
{
  Q_OBJECT

  friend class GameView;

  public:
    /** construtor of GameApp, calls all init functions to create the application.
     */
    GameApp();
    ~GameApp();
    /** enables menuentries/toolbar items
     */
    void enableAction(const char *);
    /** disables menuentries/toolbar items
     */
    void disableAction(const char *);
    /** add a opened file to the recent file list and update recent_file_menu
     */
    /** returns a pointer to the current document connected to the KMainWindow instance and is used by
     * the View class to access the document object's methods
     */	
    GameDoc *getDocument() const; 	
    void SetGrafix(QString s);

  protected:
    void NewGame();
    void EndGame();

    /** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
     * file
     */ 	
    void saveOptions();
    /** read general Options again and initialize all variables like the recent file list
     */
    void readOptions();
    /** initGUI creates the menubar and inserts the menupopups as well as creating the helpMenu.
     * @see KApplication#getHelpMenu
     */
    void initGUI();

    /**
     * builds the level submenu
     */
    void prepareLevels();

    /** this creates the toolbars.
     */
    /** sets up the statusbar for the main window by initialzing a statuslabel.
     */
    void initStatusBar();
    /** initializes the document object of the main window that is connected to the view in initView().
     * @see initView();
     */
    void initDocument();
    /** creates the centerwidget of the KMainWindow instance and sets it as the view
     */
    void initView();
    /** queryClose is called by KMainWindow on each closeEvent of a window. Against the
     * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
     * be saved if Modified; on cancel the closeEvent is rejected.
     * @see KMainWindow#queryClose
     * @see KMainWindow#closeEvent
     */
    virtual bool queryClose();
    /** queryExit is called by KMainWindow when the last window of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
     * properties.
     * @see KMainWindow#queryExit
     * @see KMainWindow#closeEvent
     */
    virtual bool queryExit();

  public slots:

    void slotRedraw();
    void slotTheme();
    void slotWallpaper();
    void slotIntro();
    void slotHelpAbout();
    /** Checks all menus..usually done on init programm */
    void checkMenus(int menu);


    /** clears the document in the actual view to reuse it as the new document */
    void slotFileNew();
    /** asks for saving if the file is modified, then closes the actual file and window*/
    void slotFileEnd();
    /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();
    // show all time stats
    void slotFileHighscores();
    // debug
    void slotFileDebug();
    /** toggles the statusbar
     */
    void slotViewStatusBar();
    /** toggles the decorations
     */
    void slotViewDecorations();
    /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);
    void slotClearStatusMsg();
    /** changes the status message of the whole statusbar for two seconds, then restores the last status. This is used to display
     * statusbar messages that give information about actions for toolbar icons and menuentries.
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusHelpMsg(const QString &text);
    /** Set the names in the mover field */
    void slotStatusNames();
    void slotStatusMover(const QString &text);
    void slotStatusTime();

    void slotLevel();
    void slotUserlevel();
    void slotTechnical();
  /** Triggers the status timer */
  void slotStatusTimer(void);

protected: // Protected attributes
  
  
  enum CheckFlags {All=0,CheckFileMenu=1,CheckOptionsMenu=2,CheckViewMenu=4,CheckEditMenu=8};
  /**  */
  /** Counts the time in the status bar */
  QTimer * statusTimer;
  QString mGrafix;

  private:
    /** the configuration object of the application */
    KConfig *config;

    GameView *view;
    /** doc represents your actual document and is created only once. It keeps
     * information such as filename and does the serialization of your files.
     */
    GameDoc *doc;
    QStringList wallpaperlist;
    QStringList mGrafixDir;
    QStringList mGrafixName;
    QString mCurrentTheme;
    QString mCurrentThemeName;
    QStringList mLevelList;
};
 
#endif 
