/***************************************************************************
                          quintaligndoc.cpp  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <assert.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qbitmap.h>
#include <qcanvas.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>


// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>

// application specific includes
#include "quintaligndoc.h"
#include "quintalign.h"
#include "quintalignview.h"
#include "highscoredlg.h"
#include "testdlg.h"

#define MAX_PATH   16383
#define REDUCE_PERCENT  35.0   /* from this free area you get less pieces if the level allows it */

GameDoc::GameDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  initRandom();
  mFirstTime=true;

  mConfig=0;
  mParent=parent;
  mDecorations=true;

  mLevelList.setAutoDelete(true);

  LevelData *data;
  
  // training
  data=new LevelData;
  data->name=i18n("Trainings level");
  data->savename=QString("0");
  data->mBoardSize=9;
  data->mNewSize=3;
  data->mFewerPieces=true;
  data->mStartPieces=3;
  data->mExtraPieces=false;
  data->mEqualProp=true;
  data->mHighJoker=true;
  data->mOriginalScore=false;
  data->mProgressiveAlign=false;
  data->mDiagonalBonus=1;
  data->mSuperJokerColor=0;
  data->mJokerColor=8;
  data->mMinAlign=5;
  data->mColors=4;
  data->mPredefined=true;
  mLevelList.append(data);


  // Align-It Easy
  data=new LevelData;
  data->name=i18n("Easy level");
  data->savename=QString("1");
  data->mBoardSize=9;
  data->mNewSize=3;
  data->mFewerPieces=true;
  data->mStartPieces=3;
  data->mExtraPieces=false;
  data->mEqualProp=true;
  data->mHighJoker=true;
  data->mOriginalScore=false;
  data->mProgressiveAlign=false;
  data->mDiagonalBonus=1;
  data->mSuperJokerColor=0;
  data->mJokerColor=8;
  data->mMinAlign=5;
  data->mColors=7;
  data->mPredefined=true;
  mLevelList.append(data);
  
  // Align-It normal level
  data=new LevelData;
  QString levelname=i18n("Normal level");
  data->savename=QString("2");
  data->name=levelname;
  data->mBoardSize=9;
  data->mNewSize=3;
  data->mFewerPieces=false;
  data->mStartPieces=7;
  data->mExtraPieces=false;
  data->mEqualProp=true;
  data->mHighJoker=false;
  data->mOriginalScore=false;
  data->mProgressiveAlign=false;
  data->mDiagonalBonus=1;
  data->mSuperJokerColor=0;
  data->mJokerColor=8;
  data->mMinAlign=5;
  data->mColors=7;
  data->mPredefined=true;
  mLevelList.append(data);
  
 
  // Advanced Level 
  data=new LevelData;
  data->name=i18n("Advanced level");
  data->savename=QString("3");
  data->mBoardSize=9;
  data->mNewSize=4;
  data->mFewerPieces=false;
  data->mStartPieces=10;
  data->mExtraPieces=false;
  data->mEqualProp=true;
  data->mHighJoker=false;
  data->mOriginalScore=false;
  data->mProgressiveAlign=false;
  data->mDiagonalBonus=0;
  data->mSuperJokerColor=0;
  data->mJokerColor=8;
  data->mMinAlign=5;
  data->mColors=7;
  data->mPredefined=true;
  mLevelList.append(data);
  
  // KLines Level 
  data=new LevelData;
  data->name=i18n("KLines level");
  data->savename=QString("4");
  data->mBoardSize=9;
  data->mNewSize=3;
  data->mFewerPieces=false;
  data->mStartPieces=3;
  data->mExtraPieces=false;
  data->mEqualProp=true;
  data->mHighJoker=false;
  data->mOriginalScore=true;
  data->mProgressiveAlign=false;
  data->mDiagonalBonus=0;
  data->mSuperJokerColor=0;
  data->mJokerColor=0;
  data->mMinAlign=5;
  data->mColors=7;
  data->mPredefined=true;
  mLevelList.append(data);
  
  
  mLevel=levelname;
  mCurrentLevel=*mLevelList.at(findLevel(mLevel));


  mChangeBoardSize=0;
  mChangeNewSize=0;
  mRunning=false;
  mIntro=true;

  mNoPathFinding=false;   // no true pathfinding
  mSpeedModify=1.0;
  mAnimModify=1.0;

  mWallpaper=0;

  generateScoreMode(); 
}

void GameDoc::generateScoreMode()
{
  unsigned int j;
  int i;
  int val,dval;
  mScoreList.clear();
  for (i=0;i<mCurrentLevel.mMinAlign;i++) mScoreList.append(0);   //0..4
  if (!mCurrentLevel.mOriginalScore) // Align-It
  {
    val=3;
    dval=2;
    for (i=mCurrentLevel.mMinAlign;i<=boardSize()+mCurrentLevel.mDiagonalBonus;i++)
    {
      mScoreList.append(val); // 5..
      val=val+dval;
      dval+=2;
    }
  }
  else  // KLines
  {
    for (i=mCurrentLevel.mMinAlign;i<=boardSize()+mCurrentLevel.mDiagonalBonus;i++)
    {
      val=2*( (i-mCurrentLevel.mMinAlign)*(i-mCurrentLevel.mMinAlign)+mCurrentLevel.mMinAlign);
      mScoreList.append(val); // 5..
    }
  }
  if (global_debug>10)
    for (j=0;j<mScoreList.count();j++) kdDebug() << "Score " << j << "="<< mScoreList[j]<<endl;



  mPropList.clear();
  mPropList.append(0);
  if (!mCurrentLevel.mEqualProp)
  {
    for (i=0;i<mCurrentLevel.mColors;i++) mPropList.append(100*(i+1)); 
    for (i=mCurrentLevel.mColors;i<mCurrentLevel.mJokerColor-1;i++) mPropList.append(0);
  }
  else
  {
    for (i=0;i<mCurrentLevel.mColors;i++) mPropList.append(100); 
    for (i=mCurrentLevel.mColors;i<mCurrentLevel.mJokerColor-1;i++) mPropList.append(0);
  }
  if (mCurrentLevel.mJokerColor!=0)
  {
    if (!mCurrentLevel.mHighJoker) mPropList.append(25);  // color 8 (50)
    else mPropList.append(40);
  }
  else mPropList.append(0);
  if (mCurrentLevel.mSuperJokerColor!=0)
  {
    if (!mCurrentLevel.mHighJoker) mPropList.append(7);  // color 9
    else mPropList.append(15);  // color 9
  }
  else mPropList.append(0);
  if (global_debug>10)
    for (j=0;j<mPropList.count();j++) kdDebug() << "Probability " << j << "="<< mPropList[j]<<endl;
}

GameDoc::~GameDoc()
{
}

void GameDoc::setView(GameView *view)
{
  pView=view;
}

void GameDoc::setAbsFilePath(const QString &filename)
{
  absFilePath=filename;
}

const QString &GameDoc::getAbsFilePath() const
{
  return absFilePath;
}

bool GameDoc::saveModified()
{
  return true;
}

bool GameDoc::newDocument(KConfig * config)
{
  mConfig=config;
  modified=false;
  absFilePath=QDir::homeDirPath();

  return true;
}

int GameDoc::random(int max)
{
  return (int)mRandom.getLong(max);
}

void GameDoc::initRandom()
{
  mRandom.setSeed(0);
  //mRandom.setSeed(global_debug);
}


// 0: all, 1:user levels, 2: predefined
void GameDoc::getLevels(QStringList &list,int mode)
{
  list.clear();
  LevelData *data;
  for ( data=mLevelList.first(); data != 0; data=mLevelList.next() )
  {
    if (mode==0) list.append(data->name);
    else if (mode==1 && !data->mPredefined) list.append(data->name);
    else if (mode==2 && data->mPredefined) list.append(data->name);
  }
}


void GameDoc::ReadConfig(KConfig *config)
{
  /*
  KConfig emailCfg( "emaildefaults", true );
  emailCfg.setGroup( "UserInfo" );
  QString name = emailCfg.readEntry( "FullName" );
  if ( name.isEmpty() )
  {
    struct passwd *pw = getpwuid( getuid() );
    if ( pw )
      name = QString::fromLatin1( pw->pw_gecos );
  }
  */

  config->setGroup("Parameter");

  mDecorations=config->readBoolEntry("decorations",mDecorations);
  mNoPathFinding=config->readBoolEntry("nopathfinding",mNoPathFinding);
  mSpeedModify=config->readDoubleNumEntry("speedmodify",mSpeedModify);
  mAnimModify=config->readDoubleNumEntry("animmodify",mAnimModify);
  mWallpaper=config->readNumEntry("wallpaper",mWallpaper);

  mLevel=config->readEntry("level",mLevel);

  readLevels(config);

  setLevel(mLevel);
  
}

void GameDoc::readLevels(KConfig *config)
{

  LevelData *data;

  // Delete all user defined levels
  for ( data=mLevelList.first(); data != 0; )
  {
    if (!data->mPredefined) 
    {
      mLevelList.remove();
      data=mLevelList.current();
    }
    else
    {
      data=mLevelList.next();
    }
  }

  QStringList levellist;
  levellist=config->readListEntry("levellist");
  // changes group from here on
  for ( QStringList::Iterator it = levellist.begin(); it != levellist.end(); ++it ) 
  {
    if (findLevel(*it)<0) // prevent overwriting of original levels
    {
      if (global_debug>5)
        kdDebug() << "load level "<<(*it)<<" " << endl;
      data=new LevelData;
      data->load(*it,config);
      mLevelList.append(data);
    }
    else
    {
      if (global_debug>5)
        kdDebug() << "Level "<<(*it)<<" predefined " << endl;
    }
  }


}

/** write config file */
void GameDoc::WriteConfig(KConfig *config)
{
  config->setGroup("Parameter");
  config->writeEntry("decorations",mDecorations);
  config->writeEntry("nopathfinding",mNoPathFinding);
  config->writeEntry("speedmodify",mSpeedModify);
  config->writeEntry("animmodify",mAnimModify);
  config->writeEntry("wallpaper",mWallpaper);

  config->writeEntry("level",mLevel);

  writeLevels(config);
  config->sync();
}

void GameDoc::writeLevels(KConfig *config)
{
  // changes group from here on
  QStringList levellist;
  LevelData *data;
  int cnt=0;
  for ( data=mLevelList.first(); data != 0; data=mLevelList.next() )
  {
    if (!data->mPredefined)
    {
      levellist.append(data->name);
      data->save(config);
    }
    cnt++;
  }

  config->setGroup("Parameter");
  config->writeEntry("levellist",levellist);

  config->sync();
}



// ------------- Game specific ------------------
void GameDoc::startIntro()
{
  pView->drawIntro();
}
void GameDoc::deleteAllPieces()
{
  pView->drawGameOver(0,false,true);
  pView->drawScore(true);
  pView->drawBoard(boardSize(),newSize(),true);
  pView->drawDecoration(true);
  for (int y=0;y<boardSize();y++)
  {
    for (int x=0;x<boardSize();x++)
    {
      int col=getBoard(x,y);
      if (col!=0)
      {
        int no=mNumber.at(aIndex(x,y));
        PSprite *sprite=(PSprite *)setBoard(x,y,0,0,no);
        if (sprite)
        {
          pView->deleteItem(sprite);
          mColour.at(no)=0;
          mNumber.at(aIndex(x,y))=0;
          mBoard.at(aIndex(x,y))=0;
        }
      }
    }
  }
  for (int i=0;i<newSize();i++)
  {
    int col=getNew(i);
    if (col!=0)
    {
      int no=mNewNumber.at(i);
      PSprite *sprite=(PSprite *)setNew(i,0,0,no);
      if (sprite)
      {
        pView->deleteItem(sprite);
        mColour.at(no)=0;
        mNewNumber.at(i)=0;
        mNew.at(i)=0;
      }
    }
  }
}

void GameDoc::initBoard()
{
  unsigned int i;
  if (!mFirstTime) deleteAllPieces();

  if (mChangeBoardSize)
  {
    mCurrentLevel.mBoardSize=mChangeBoardSize;
    mChangeBoardSize=0;
  }
  if (mChangeNewSize)
  {
    mCurrentLevel.mNewSize=mChangeNewSize;
    mChangeNewSize=0;
  }
  pView->initView();
  
  int size,newsize;
  newsize=mCurrentLevel.mNewSize;
  size=mCurrentLevel.mBoardSize;

  generateScoreMode(); 

  // Calculate the probability list (do here as it can change from game to game!)
  mMaxProp=0;
  for (unsigned int i=0;i<mPropList.count();i++)
  {
    mMaxProp+=mPropList[i];
  }

  mRemoved.resize(mCurrentLevel.mColors+1);
  mColour.resize(size*size+newsize+25); // make sure
  mNumber.resize(size*size);
  mNewNumber.resize(newsize);
  mBoard.resize(size*size);
  mPath.resize(size*size);
  mNew.resize(newsize);
  for (i=0;i<mRemoved.size();i++) mRemoved.at(i)=0;
  for (i=0;i<mColour.size();i++) mColour.at(i)=0;
  for (i=0;i<mBoard.size();i++) mBoard.at(i)=0;
  for (i=0;i<mNumber.size();i++) mNumber.at(i)=0;
  for (i=0;i<mNew.size();i++) mNew.at(i)=0;
  for (i=0;i<mNewNumber.size();i++) mNewNumber.at(i)=0;
  mScore=0; 
  mMoveFirstX=-1;
  mMoveFirstY=-1;
  mMoveSecondX=-1;
  mMoveSecondY=-1;
  mPieceCount=0;
  mMoveStatus=NoMove;
  mFirstTime=false;
  mMoves=0;



  mRunning=true;
}

void GameDoc::endGame()
{
  if (!mRunning) return ;
  PSprite *sprite;
  sprite=(PSprite *)pView->drawGameOver();
  mRunning=false;
  
  if (!sprite) // if game over cannot be displayed
  {
    showHighscores(score());
  }
  else
  {
    QObject *ob=sprite->emitter(true);
    connect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
            this,SLOT(slotGameOverMove(QCanvasItem *,int)));
  }

  if (global_debug>15)
  {
    kdDebug() << "Removed pieces " << endl;
    for (unsigned int i=0;i<mRemoved.count();i++)
    {
      kdDebug() << "Color " << (i+1) << " " << mRemoved.at(i) << " removed" << endl;
    }
  }
}
void GameDoc::slotGameOverMove(QCanvasItem *item,int mode)
{
  if (global_debug>10)
    kdDebug() << "GameDoc::slotGameOverMove  mode="<<mode<<endl;
  if (!item) return ;
  PSprite *sprite=(PSprite *)item;
  if (mode!=1) return ;
  
  QObject *ob=sprite->emitter(false);
  if (ob) disconnect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
                     this,SLOT(slotGameOverMove(QCanvasItem *,int)));
  sprite->deleteEmitter();
  showHighscores(score());
} 
void GameDoc::newGame()
{
  if (global_debug>2)
    kdDebug() << "++++ New game level= " << mLevel << endl;
  mIntro=false;
  setLevel(mLevel); // init defaults
  initBoard();
  pView->drawScore(false);
  pView->drawBoard(boardSize(),newSize(),false);
  pView->drawDecoration(!mDecorations);

  if (global_debug>5)
    kdDebug() << "new Game with "<<mCurrentLevel.mStartPieces<<" pieces" << endl;

  for (int i=0;i<mCurrentLevel.mStartPieces;i++)
  {
    int x,y;
    getRandomPosition(x,y);
    int col=getRandomPiece();
    // kdDebug() << "Draing start piece " << i << " as color " << col << endl;
    setBoard(x,y,col,3);
  }
  
  for (int i=0;i<newSize();i++)
  {
    int col=getRandomPiece();
    setNew(i,col,2);
  }
  mNewPieces=newSize();
  
}

int GameDoc::getBoard(int x, int y)
{
  if (x>=boardSize() || y>=boardSize())
  {
    kdError() << "GameDoc::getBoard:: Try to access piece outside board" << endl;
    return 0;
  }
  assert(aIndex(x,y)<(int)mBoard.count());
  return mBoard.at(aIndex(x,y));
}

// no=-1: new piece
QCanvasItem * GameDoc::setBoard(int x, int y,int col,int animated,int no)
{
  if (x>=boardSize() || y>=boardSize())
  {
    kdError() << "GameDoc::setBoard:: Try to access piece outside board" << endl;
    return 0;
  }
  // Find a free piece
  if (no<0)
  {
    for (unsigned int i=0;i<mColour.size()&&no<0;i++)
    {
      if (mColour.at(i)==0) no=i;
    }
    if (no<0)
    {
      kdDebug() << "ERROR: Could not find a free sprite number in setBoard!!!"<<endl;
    }
    mColour.at(no)=1;
    mPieceCount++;
    // kdDebug() << "setBoard assignes new number="<<no<<endl;
  }
  assert(aIndex(x,y)<(int)mNumber.count());
  mNumber.at(aIndex(x,y))=no;
  PSprite *sprite;
  if (col!=0)
  {
    sprite=(PSprite *)pView->drawPiece(x,y,col,no,0);
  }
  else
  {
    int col2=mBoard.at(aIndex(x,y));
    QString s=QString("piece%1").arg(col2);
    if (col2) sprite=(PSprite *)(pView->getItem(s,no));
    else sprite=0;
  }
  if (animated && sprite) sprite->setAnimation(animated,mAnimModify);
  if (!sprite) kdDebug() << "ERROR: set board has no sprite!!!!" <<endl;
  mBoard.at(aIndex(x,y))=col;
  return sprite;
}

int GameDoc::getNew(int x)
{
  if (x>=newSize())
  {
    kdError() << "GameDoc::getNew:: Try to access piece outside board" << endl;
    return 0;
  }
  return mNew.at(x);
}
// no=-1 new piece
QCanvasItem * GameDoc::setNew(int y,int col,int animated,int no)
{
//  kdDebug() << "setNew y="<<y<<" col="<<col<<" anim="<<animated<<" no="<<no<<endl;
  if (y>=newSize())
  {
    kdError() << "GameDoc::setNew:: Try to access piece outside board" << endl;
    return 0;
  }
  // Find a free piece
  if (no<0)
  {
    for (unsigned int i=0;i<mColour.size()&&no<0;i++)
    {
      if (mColour.at(i)==0) no=i;
    }
    assert(no<(int)mColour.count());
    mColour.at(no)=1;
    if (no<0)
    {
      kdDebug() << "ERROR: Could not find a free sprite number in setNew!!!"<<endl;
    }
  }
  mNewNumber.at(y)=no;
  PSprite *sprite;
  if (col!=0)
  {
    sprite=(PSprite *)pView->drawPiece(0,y,col,no,1);
  }
  else
  {
    int col2=mNew.at(y);
    QString s=QString("piece%1").arg(col2);
    if (col2) sprite=(PSprite *)(pView->getItem(s,no));
    else sprite=0;
  }
  if (animated && sprite) sprite->setAnimation(animated,mAnimModify);
  if (!sprite) kdDebug() << "ERROR: set board has no sprite!!!!" <<endl;
  mNew.at(y)=col;
  return sprite;
}

int GameDoc::gameStatus()
{
  return 0;
}

void GameDoc::firstPiece(int x ,int y)
{
  QString s;
  // kdDebug() << "First click " << x << " " << y << endl;

  // We have to overide a former move
  if (mMoveFirstX>=0 && mMoveFirstY>=0)
  {
    s=QString("piece%1").arg((int)mMoveCol);
    PSprite *sprite=(PSprite *)(pView->getItem(s,mMoveSprite));
    sprite->setAnimation(0);
  }
  setMoveStatus(MoveStart);
  mMoveFirstX=x;
  mMoveFirstY=y;
  mMoveCol=mBoard.at(aIndex(x,y));
  mMoveSprite=mNumber.at(aIndex(x,y));
  s=QString("piece%1").arg((int)mMoveCol);
  PSprite *sprite=(PSprite *)(pView->getItem(s,mMoveSprite));
  sprite->setAnimation(1,mAnimModify);
}

bool GameDoc::secondPiece(int x ,int y)
{
  if (!calcPath(mMoveFirstX,mMoveFirstY,x,y)) return false;
  // kdDebug() << "Second click " << x << " " << y << endl;
  mBoard.at(aIndex(mMoveFirstX,mMoveFirstY))=0;
  mNumber.at(aIndex(mMoveFirstX,mMoveFirstY))=0;
  int tmpx,tmpy;
  tmpx=mMoveFirstX;
  tmpy=mMoveFirstY;
  mMoveFirstX=-1;
  mMoveFirstY=-1;
  mMoveSecondX=x;
  mMoveSecondY=y;
  setMoveStatus(MoveEnd);
  
  return true;
}

bool GameDoc::moveFinished()
{
    // kdDebug() << "Destination "<<mMoveSecondX << " " << mMoveSecondY << " reached " << endl;
    setBoard(mMoveSecondX,mMoveSecondY,mMoveCol,0,mMoveSprite);
    setMoveStatus(PrepareNew);
 
    // check whether a) game over b) remove pieces c) insert pieces
    int newpieces=calcNumberNewPieces();

    mMoves++;
    QString s=i18n("Quintalign %1: %2 moves").arg(mCurrentLevel.name).arg(mMoves);
    emit statusMove(s);
    // check for five pieces
    int align;
    //align=checkForFive(mMoveSecondX,mMoveSecondY,0);
    align=checkForFive2(mMoveSecondX,mMoveSecondY,0);
    if (align)
    {
      //kdDebug() << "FIVE FOUND (align="<<align<<")!!!!!!!!!!"<<endl;
      // remove pieces
      //int sc=checkForFive(mMoveSecondX,mMoveSecondY,align);
      int sc=checkForFive2(mMoveSecondX,mMoveSecondY,align);
      mScore+=sc;
      pView->drawScore();
    }
    else if (mPieceCount<1)
    {
      insertNewPieces(mNewPieces,newpieces);
      mNewPieces=newpieces;
    }
    else
    {
      // insert new pieces
      if (!insertNewPieces(mNewPieces,newpieces))
      {
        if (global_debug>10)
          kdDebug() << " GAME OVER " << endl;
        QString gameover=i18n("Game over...restart game");
        emit statusMove(gameover);
        endGame();
      }
      else
      {
        mNewPieces=newpieces;
      }
    }
    setMoveStatus(NoMove);
    
    return true;
}

bool GameDoc::isJoker(int col)
{
  if (mCurrentLevel.mJokerColor!=0 && col==mCurrentLevel.mJokerColor) return true;
  return false;
}
bool GameDoc::isSuperJoker(int col)
{
  if (mCurrentLevel.mSuperJokerColor!=0 && col==mCurrentLevel.mSuperJokerColor) return true;
  return false;
}


int GameDoc::checkForPiece(int &col,int x,int y,int &count,int align)
{
  //kdDebug() << "check for piece col="<<col<<" x="<<x << "y="<<y<<" align="<<align<<endl;
  int color=getBoard(x,y);
  if (color==0) return false; // empty

  if (isJoker(col) && !isJoker(color)) col=color;

  if (color==col || isJoker(color))
  {
    count++;
    if (align) removePiece(x,y);
  }
  else
  {
    return false;
  }
  return true;
}

int GameDoc::getLine(int x,int y,int &col,int dx,int dy,int remove,int &jokerlen)
{
  int len=1;
  bool keeprunning=true;
  int curcol;
  bool isjoker;

  col=getBoard(x,y);
  if (!col) return 0;
  isjoker=isJoker(col)||isSuperJoker(col);
  jokerlen=0;

  do
  {
    x+=dx;
    y+=dy;
    if(x>=0 && y>=0 && x<mCurrentLevel.mBoardSize && y<mCurrentLevel.mBoardSize)
    {
      curcol=getBoard(x,y);
      if (curcol)
      {
        // check how many pure joker
        if ((isJoker(col)||isSuperJoker(col)) && (isJoker(curcol) || isSuperJoker(curcol)) )
        {
          jokerlen++;
          if (remove&0xff00) removePiece(x,y);
        }
        if ( curcol==col || isJoker(curcol) || isSuperJoker(curcol) )
        {
          len++;
          if (remove&0xff) removePiece(x,y);
        }
        else if (isJoker(col) || isSuperJoker(col))
        {
          col=curcol;
          len++;
          if (remove&0xff) removePiece(x,y);
        }
        else
        {
          keeprunning=false;
        }
      }
      else
      {
        keeprunning=false;
      }
    }
    else
    {
      keeprunning=false;
    }
  }while(keeprunning);
  return len;
}

int GameDoc::calcLen(int len1,int len2,bool samecol,int &resultalign,int val1,int val2,int jlen1,int jlen2)
{
  int minAlign=mCurrentLevel.mMinAlign;
  int len=1;

  if (!samecol)
  {
    if (len1>=minAlign && len2>=minAlign)
    {
      len=len1+len2-1;
      resultalign|=val1|val2;
    }
    else if (len1+jlen2>=minAlign)
    {
      len=len1+jlen2;
      resultalign|=val1 | (val2<<8);
    }
    else if (len1>=minAlign)
    {
      len=len1;
      resultalign|=val1;
    }
    else if (len2+jlen1>=minAlign)
    {
      len=len2+jlen1;
      resultalign|=val2 | (val1<<8);
    }
    else if (len2>=minAlign)
    {
      len=len2;
      resultalign|=val2;
    }
  }
  else if (len1+len2-1>=minAlign)
  {
    len=len1+len2-1;
    resultalign|=val1|val2;
  }
  return len;
}

int GameDoc::checkForFive2(int x,int y,int align)
{
  int minAlign=mCurrentLevel.mMinAlign;
  int runcol1,runcol2;
  int len,len1,len2;
  int jlen1,jlen2;
  int score,resultalign;
  score=0;
  resultalign=0;


  // Horizontal
  len1=getLine(x,y,runcol1,1,0,align&0x0101,jlen1);
  len2=getLine(x,y,runcol2,-1,0,align&0x0202,jlen2);
  len=calcLen(len1,len2,runcol1==runcol2,resultalign,0x01,0x02,jlen1,jlen2);
  if (len>=minAlign) score+=calcScore(len,0x03); 

  // Vertical
  len1=getLine(x,y,runcol1,0,1,align&0x0404,jlen1);
  len2=getLine(x,y,runcol2,0,-1,align&0x0808,jlen2);
  len=calcLen(len1,len2,runcol1==runcol2,resultalign,0x04,0x08,jlen1,jlen2);
  if (len>=minAlign) score+=calcScore(len,0x0c);

  // Diagonal 1
  len1=getLine(x,y,runcol1,1,1,align&0x1010,jlen1);
  len2=getLine(x,y,runcol2,-1,-1,align&0x2020,jlen2);
  len=calcLen(len1,len2,runcol1==runcol2,resultalign,0x10,0x20,jlen1,jlen2);
  if (len>=minAlign) score+=calcScore(len,0x30); 


  // Diagonal 2
  len1=getLine(x,y,runcol1,1,-1,align&0x4040,jlen1);
  len2=getLine(x,y,runcol2,-1,1,align&0x8080,jlen2);
  len=calcLen(len1,len2,runcol1==runcol2,resultalign,0x40,0x80,jlen1,jlen2);
  if (len>=minAlign) score+=calcScore(len,0xc0); 

  // Remove central piece
  if (align) removePiece(x,y);

  // Return result
  if (align) return score;
  else return resultalign;
}

// direction: 0x03: x, 0xc0: y, 0x30:diagonal TL-BR, 0xc0:diagonalBL-TR
int GameDoc::calcScore(unsigned int count,int direction)
{
  if (direction&0xf0) count+=mCurrentLevel.mDiagonalBonus;
  assert(count<mScoreList.count());
  if (count>=mScoreList.count())
  {
    kdDebug() << "Warning: Score for "<<count<<" pieces not defined"<<endl;
    count=mScoreList.count()-1;
  }
  return mScoreList[count];
}

int GameDoc::calcNumberNewPieces()
{
  int no;
  if (!mCurrentLevel.mFewerPieces) return newSize();


  int p_free=boardSize()*boardSize()-mPieceCount;
  float percent=100.0*(float)p_free/(float)(boardSize()*boardSize());
//  kdDebug() << "Few pieces: percent="<<percent<<" free="<<p_free<<endl;
  if (percent>REDUCE_PERCENT) return newSize();
  no=(int)((REDUCE_PERCENT-percent)/10.0);
  if (newSize()-no>1) return newSize()-no;
  return 1;
}

void GameDoc::getRandomPosition(int &x,int &y)
{
  int rnd=random(boardSize()*boardSize()-mPieceCount);
  int cnt=0;
  for (int yy=0;yy<boardSize();yy++)
  {
    for (int xx=0;xx<boardSize();xx++)
    {
      if (getBoard(xx,yy)==0) cnt++;
      if (cnt>rnd)
      {
        x=xx;
        y=yy;
        return ;
      }
    }
  }
  x=0;
  y=0;
  kdDebug() << "WARNING: getRandomposition cannot find empty field!!!"<<endl;
}

void GameDoc::removePiece(int x,int y)
{
    int col=getBoard(x,y);
    // but remove all new ones
    int no=mNumber.at(aIndex(x,y));
    //kdDebug() << "removePiece("<<x<<","<<y<<") col="<<col<<" no="<<no<<endl;
    if (col==0)
    {
      kdDebug() << "Why do we want to remove an empty field ("<<x<<","<<y<<")?"<<endl;
      return ;
    }
    // Log how many pieces of each color got removed
    assert(col-1>=0 && col-1<(int)mRemoved.count());
    mRemoved.at(col-1)+=1;
    //kdDebug() << "remove piece x="<<x<<" y="<<y<<" col="<<col<<" no="<<no<<endl;
    PSprite *sprite=(PSprite *)setBoard(x,y,0,4,no);
    if (sprite)
    {
      sprite->setData(no);
      QObject *ob=sprite->emitter(true);
      connect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
              this,SLOT(slotPieceRemoved(QCanvasItem *,int)));
      //kdDebug() << "Connecting emmiter " << ob << "  of " << sprite << endl;
    }
    else
    {
      kdDebug() << "ERROR: Could not find piece in remove "<<endl;
    }
    mPieceCount--;
}

bool GameDoc::gameOver()
{
  if (mPieceCount>=boardSize()*boardSize()) return true;
  return false;
}

void GameDoc::redrawNewPieces()
{
   insertNewPieces(mNewPieces,mNewPieces,true);
}

bool GameDoc::insertNewPieces(int oldcount,int newcount,bool replace)
{
  int i,col,x,y,no;
  QString s;
  PSprite *sprite;
//  kdDebug() << "GameDoc::insertNewPieces old="<<oldcount<<", new=" << newcount<<endl;

  for (i=0;i<newSize();i++)
  {
    // remove and put onto board
    if (i<oldcount)
    {
      col=getNew(i);
      if (col && !replace)
      {
        getRandomPosition(x,y);
        /*
        TestDlg dlg;
        dlg.LineEdit1->setText(QString("%1").arg(x));
        dlg.LineEdit3->setText(QString("%1").arg(y));
        dlg.exec();
        QString sx=dlg.LineEdit1->text();
        QString sy=dlg.LineEdit3->text();
        x=sx.toShort();
        y=sy.toShort();
        */
        // kdDebug() << " New piece " << i << " col=" << col << " on " << x << " " << y << endl; 


        sprite=(PSprite *)setBoard(x,y,col,3);
        //kdDebug() << "New board sprite="<<sprite<<endl;
      
        // Check for autoremove
        int align=checkForFive2(x,y,0);
        if (align)
        {
          //kdDebug() << "AUTO FIVE FOUND (align="<<align<<")!!!!!!!!!!"<<endl;
          // remove pieces
          int sc=checkForFive2(x,y,align);
          mScore+=sc;
          pView->drawScore();
        }
      }
      else if (!col)
      {
        kdDebug()  << "ATTENTION: insertNew Pieces color("<<i<<")=000" << endl;
      }
    
      // but remove all new ones
      no=mNewNumber.at(i);
      sprite=(PSprite *)setNew(i,0,0,no);
      // sprite=(PSprite *)setNew(i,0,2,no);
      if (sprite)
      {
        sprite->setZ(sprite->z()-10.0);
        sprite->setData(no);
        /*
        connect(sprite->emitter(),SIGNAL(signalNotify(QCanvasItem *,int)),
                this,SLOT(slotPieceInserted(QCanvasItem *,int)));
        */
        slotPieceInserted(sprite,2); // direct call gives ncier animation
      }
      else
      {
        kdDebug() << "ERROR: Could not find new piece "<<s<<endl;
      }
    }

    no=mNewNumber.at(i);
    // kdDebug() << "Number ("<<i<<")="<<no<<endl;
     
    // not sure it is right here..
    if (gameOver()) return false;
   
    // and create the new pieces on the spare board
    if (i<newcount)
    {
      col=getRandomPiece();
      if (col==0) 
      {
        kdDebug() << "THIS SHOULD NOT HAPPEN...piece with color 0 created!!!"<<endl;
      }
      setNew(i,col,2);
      //kdDebug() << "creating new one with color "<<col<<endl;
      // We can release the number of the remove piece only after
      // we inserted thenew ones!!
      //mColour.at(no)=0;
    }

  }
  return true;
}

int GameDoc::getRandomPiece()
{
 assert(mMaxProp!=0);
 int rnd=random(mMaxProp);
 int cnt=0;
 int col=0;
 for (unsigned int i=0;i<mPropList.count();i++)
 {
   cnt+=mPropList[i];
   if (cnt>rnd)
   {
     col=i;
     break;
     //return (int)(i);
   }
 }
 if (col==0)
 {
   kdDebug() << "Warning: We could not calc a proper color="<<col<<" maxprop="<<mMaxProp<<" rnd="<<rnd<<endl;
 }
 if (col>8)
 {
   kdDebug() << "Warning: We could not calc a proper color="<<col<<" maxprop="<<mMaxProp<<" rnd="<<rnd<<endl;
 }
 assert(col!=0);
 return col;
}

void GameDoc::slotPieceInserted(QCanvasItem *item,int mode)
{
//  kdDebug() << "GameDoc::slotPieceInserted  mode="<<mode<<endl;
  if (!item) return ;
  PSprite *sprite=(PSprite *)item;
  if (mode!=2) return ;
  int no=sprite->data();
  
  
  /*
  disconnect(sprite->emitter(),SIGNAL(signalNotify(QCanvasItem *,int)),
            this,SLOT(slotPieceInserted(QCanvasItem *,int)));
  sprite->deleteEmitter();
  */
  pView->deleteItem(sprite);
  mColour.at(no)=0;
} 

void GameDoc::slotPieceRemoved(QCanvasItem *item,int mode)
{
   //kdDebug() << "GameDoc::slotPieceRemoved  mode="<<mode<<endl;
  if (!item) return ;
  PSprite *sprite=(PSprite *)item;
  if (mode!=2) return ;
  int no=sprite->data();
  
  QObject *ob=sprite->emitter(false);
  if (ob) disconnect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
                     this,SLOT(slotPieceRemoved(QCanvasItem *,int)));
  sprite->deleteEmitter();
  pView->deleteItem(sprite);
  mColour.at(no)=0;
} 


ListData *GameDoc::waypoint()
{
  ListData *waypoint=mMoveList.first();
  return waypoint;
}
void GameDoc::removeWaypoint(ListData *waypoint)
{
  if (!waypoint) return ;
  mMoveList.remove(waypoint);
}


// ------------------- pathfinding -------------------------
bool GameDoc::calcPath(int x0,int y0,int x1,int y1)
{
  // Clear map
  int i;
  for (i=0;i<boardSize()*boardSize();i++) mPath.at(i)=MAX_PATH;

  //kdDebug() << " Prepare pathfinding" << endl;
  // Put shortest path in list
  mMoveList.clear();
  ListData *waypoint;

  // Cheat
  if (mNoPathFinding)
  {
    kdDebug() << "Warning: No pathfinding enabled!!!!"<<endl;
    waypoint=new ListData;
    waypoint->x=x1;
    waypoint->y=y1;
    waypoint->level=1;
    mMoveList.append(waypoint);
    return true;
  }

  // calulate path gradient map
  addList(x1,y1,0);
  fillPath(x0,y0);

  /*
  // Debug output
  printf("Pathgradientmap:\n");
  for (int y=0;y<mBoardSize;y++)
  {
    for (int x=0;x<mBoardSize;x++)
    {
      printf("%02x ",mPath.at(aIndex(x,y)));
    }
    printf("\n");
  }
  */

  // calculate path
  assert(aIndex(x0,y0)>=0 && aIndex(x0,y0)<(int)mPath.count());
  int value=mPath.at(aIndex(x0,y0));

  int x,y;
  int v;
  bool found;
  do
  {
    found=false;
    x=x0;
    y=y0;
    v=calcMinNeighbour(x0-1,y0,value);
    if (v<value)
    {
      value=v;
      x=x0-1;
      y=y0;
      found=true;
    }
    v=calcMinNeighbour(x0+1,y0,value);
    if (v<value)
    {
      value=v;
      x=x0+1;
      y=y0;
      found=true;
    }
    v=calcMinNeighbour(x0,y0-1,value);
    if (v<value)
    {
      value=v;
      x=x0;
      y=y0-1;
      found=true;
    }
    v=calcMinNeighbour(x0,y0+1,value);
    if (v<value)
    {
      value=v;
      x=x0;
      y=y0+1;
      found=true;
    }
    x0=x;
    y0=y;
    waypoint=new ListData;
    waypoint->x=x0;
    waypoint->y=y0;
    waypoint->level=value;
    mMoveList.append(waypoint);
  }while((x0!=x1 || y0!=y1) && value<MAX_PATH && found);

  if (!found)
  {
    kdDebug() << "  No path found -- maybe problem with the pathfinding algorithm" << endl;
    return false;
  }
  if (value==MAX_PATH)
  {
    kdDebug() << "  No path found" << endl;
    return false;
  }

  /*
  kdDebug() << "MOVE PATH:"<<endl;
  for ( waypoint=mMoveList.first(); waypoint != 0; waypoint=mMoveList.next() )
  {
    kdDebug() << " Move: " << waypoint->x << " " << waypoint->y << " " << waypoint->level << endl;
  }
  */

  return true;
}

int GameDoc::calcMinNeighbour(int x, int y,int value)
{
  if (x<0 || y<0 || x>=boardSize() || y>=boardSize()) return MAX_PATH;
  int newvalue=mPath.at(aIndex(x,y));
  if (newvalue<value) return newvalue;
  else return MAX_PATH;
  
}

MList::MList() : QPtrList<ListData>()
{
  setAutoDelete(true);
}
int MList::compareItems( QPtrCollection::Item item1, QPtrCollection::Item item2)
{
  ListData *a=(ListData *)item1;
  ListData *b=(ListData *)item2;
  if (a->level>b->level) return 1;
  else if (a->level<b->level) return -1;
  return 0;
}


void GameDoc::addList(int x,int y,int level,bool force)
{
  if (x<0 || y<0 || x>=boardSize() || y>=boardSize()) return ;
  if (mPath.at(aIndex(x,y))<=level) return ;
  if (!force && mBoard.at(aIndex(x,y))!=0) return ;
  ListData *a;
  a=new ListData;
  a->x=x;
  a->y=y;
  a->level=level;
  mList.inSort(a);
  //kdDebug() << "addList x="<<x<<" y="<<y<<" level="<<level<<" force="<<force<<endl;
  //kdDebug() << "addList done"<<endl;

  /*
  for ( a=mList.first(); a != 0; a=mList.next() )
  {
    printf("%2d -> ",a->level);
  }
  printf("\n");
  */
}

bool GameDoc::fillPath(int x0,int y0)
{
  int x,y;
  int level;
  // kdDebug() << " fill path List cnt=" << mList.count() << endl;
  if (mList.count()<=0) return false;
  ListData *first=mList.first();
  x=first->x;
  y=first->y;
  level=first->level;
  mList.removeFirst();
  if (x==x0 && y==y0)
  {
    // This does not work cause the path following algorithm is too
    // stupid and runs into dead ends
    // kdDebug() << "TARGET EARLY FOUND"<<endl;
    // return true;
  }
//  kdDebug() << "Current Pathpoint to branch from  x="<<x<<" y="<<y<<" level="<<level<<" x0="<<x0<<" y0="<<y0<<endl;
  
  int pathlevel=mPath.at(aIndex(x,y));
  if (pathlevel>level)
  {
    mPath.at(aIndex(x,y))=level;
    if (x-1==x0 && y==y0) addList(x-1,y,level+1,true);
    else addList(x-1,y,level+1);
    if (x+1==x0 && y==y0) addList(x+1,y,level+1,true);
    else addList(x+1,y,level+1);
    if (x==x0 && y-1==y0)  addList(x,y-1,level+1,true);
    else addList(x,y-1,level+1);
    if (x==x0 && y+1==y0) addList(x,y+1,level+1,true);
    else addList(x,y+1,level+1);
  }
  else
  {
    //kdDebug() << " we do not want to process this one cause mPath.at()="<<pathlevel<<" and level="<<level<<endl;
  }
  return fillPath(x0,y0);
}
  
void GameDoc::Debug()
{
  pView->Debug();
  if (!isIntro())
  {
    printf("Boardmap:\n");
    for (int y=0;y<boardSize();y++)
    {
      for (int x=0;x<boardSize();x++)
      {
        printf("%02x ",mBoard.at(aIndex(x,y)));
      }
      printf("\n");
    }
    printf("Piececount=%d\n",mPieceCount);
    printf("Newmap:\n");
    for (int i=0;i<newSize();i++)
    {
      printf("%02x ",mNew.at(i));
    }
    printf("\n");
  }
}

// Fill/read dialogs
void GameDoc::setTechDlg(TechDlg *dlg)
{
  dlg->setData(mAnimModify,mSpeedModify);
}
void GameDoc::getTechDlg(TechDlg *dlg)
{
  dlg->getData(mAnimModify,mSpeedModify);
}

int GameDoc::findLevel(QString name)
{
  int cnt=0;
  LevelData *data;
  for ( data=mLevelList.first(); data != 0; data=mLevelList.next() )
  {
    if (name==data->name) return cnt;
    cnt++;
  }
  return -1;

}

void GameDoc::setLevel(QString level)
{
    int no=findLevel(level);
    if (no>=0)
    {
      mLevel=level;
      mCurrentLevel=*mLevelList.at(findLevel(mLevel));
      if (global_debug>10)
        kdDebug() << "set current level to  " << level << " name=" << mCurrentLevel.name<< endl;
    }
}


void GameDoc::setBackground(QPixmap &background)
{
  if (pView) pView->setBackground(background);
}

void GameDoc::showHighscores(int score)
{
  HighScoreDlg *dlg=new HighScoreDlg(mParent,"highscoredlg");
  dlg->enterName(mConfig,mCurrentLevel.name,mCurrentLevel.savename,score); 
  if (dlg->exec()==QDialog::Accepted)
  {
    dlg->saveData();
  }
  delete dlg;
  emit gameOver(0);
//  ((GameApp *)mParent)->checkMenus();
}


bool GameDoc::decoration()
{
  return mDecorations;
}

void GameDoc::setDecoration(bool show)
{
  if (global_debug>3)
    kdDebug() << "+++++GameDoc::setdecoration="<<show<<endl;
  mDecorations=show;
  if (!isIntro()) pView->drawDecoration(!mDecorations);
}

void GameDoc::changeTheme(QString theme)
{
  if (global_debug>3)
    kdDebug() << "+++++GameDoc::changetheme="<<theme<<endl;
  pView->changeTheme(theme,mDecorations);
  
  // No board for intro
  if (isIntro())
  {
    startIntro();
    return ;
  }

  pView->drawScore(false);
  pView->drawBoard(boardSize(),newSize(),false);
  if (mDecorations) pView->drawDecoration(!mDecorations);

  int i,col,no,x,y;
  // redraw pieces
  for (i=0;i<newSize();i++)
  {
      col=getNew(i);
      no=mNewNumber.at(i);
      //kdDebug()  << "setNew("<<i<<","<<col<<","<<2<<","<<no<<")"<<endl;
      if (col) setNew(i,col,2,no);
  }
  for (y=0;y<boardSize();y++)
  {
    for (x=0;x<boardSize();x++)
    {
      col=getBoard(x,y);
      no=mNumber.at(aIndex(x,y));
      //kdDebug()  << "setBoard("<<x<<","<<y<<","<<col<<","<<3<<","<<no<<")"<<endl;
      if (col) setBoard(x,y,col,3,no);
    }
  }

  // Piece animated?
  if (mMoveFirstX>=0 && mMoveFirstY>=0)
  {
    firstPiece(mMoveFirstX,mMoveFirstY);
  }
  if (!isRunning() ) pView->drawGameOver(0,false,false);
}

bool GameDoc::noRedraw()
{
  return !mCurrentLevel.mExtraPieces;
}


