/***************************************************************************
                          quintaligndoc.h  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUINTALIGNDOC_H
#define QUINTALIGNDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qptrcollection.h>
#include <qmap.h>
#include <qpixmapcache.h>
#include <qtimer.h>
#include <qcanvas.h>

#include <kconfig.h>
#include <kdebug.h>
#include <krandomsequence.h>

#include "quintalign.h"
#include "techdlg.h"
#include "leveldata.h"


// forward declaration of the Game classes
class GameView;

/**
 * List item of the waypoint list
 */
class ListData
{
  public:
    int x,y;
    int level;
};

/**
 * QPtrList - needed to define the compareItems function 
 * and be able to sort the list
 */
class MList:public QPtrList<ListData>
{
  public:
    MList();
    int compareItems( QPtrCollection::Item item1, QPtrCollection::Item item2);
};

/**	GameDoc provides a document object for a document-view model.
  *
  * The GameDoc class provides a document object that can be used in conjunction with the classes GameApp and GameView
  * to create a document-view model for standard KDE applications based on KApplication and KTMainWindow. Thereby, the document object
  * is created by the GameApp instance and contains the document structure with the according methods for manipulation of the document
  * data by GameView objects. Also, GameDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 0.4 code generation
  */
class GameDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    GameDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~GameDoc();

    /**
     * Outputs debug inforamtion
     */
    void Debug();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void setView(GameView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** "save modified" - asks the user for saving if the document is modified */
    bool saveModified();	
    /** initializes the document generally */
    bool newDocument(KConfig *config);
    /** sets the path to the file connected with the document */
    void setAbsFilePath(const QString &filename);
    /** returns the pathname of the current document file*/
    const QString &getAbsFilePath() const;

  void setBackground(QPixmap &background);
  void setWallpaper(int no) {mWallpaper=no;}
  int wallpaper()  {return mWallpaper;}
  void setDecoration(bool show);
  bool decoration();


   int random(int max);

   void WriteConfig(KConfig *config);
   void ReadConfig(KConfig *config);
   void writeLevels(KConfig *config);
   void readLevels(KConfig *config);
   
  void setTechDlg(TechDlg *);
  void getTechDlg(TechDlg *);
  void setLevel(QString level);
  QString level() {return mLevel;}
  void getLevels(QStringList &list,int mode);

  void startIntro();
  void showHighscores(int score=0);


   // --- Game specific ---
   /**
    * set up game specific stuff
    *
    */
  void generateScoreMode();
   /**
    * Initializes the board with size (size,size) and 
    * the size of the new list (newlist)
    */
   void initBoard();
   /**
    * gets the piece at (x,y)
    */
   int getBoard(int x,int y);
   /**
    * Gets the new pieces at x
    */
   int getNew(int x);
   /**
    * Sets the new pieces at x
    */
   QCanvasItem * setNew(int x,int col,int animated=0,int no=-1);
   /**
    * Sets a piece onto the board or removes it for col=0
    */
   QCanvasItem * setBoard(int x0, int y0,int col,int animated=0,int no=-1);
   /**
    * Checks the game status, e.g. game over
    */
   int gameStatus();

  /**
   * Loads all game bitmaps into the memory
   */
//  QPixmap *getPixmap(QString name);

  /**
   * Get the size of the board
   */
  int boardSize() {return mCurrentLevel.mBoardSize;}

  /**
   * Get the size of the new field
   */
  int newSize() {return mCurrentLevel.mNewSize;}

  /**
   * Get the size of the new field
   */
  int score() {return mScore;}
  /**
   * How many pieces are on the board
   */
  int pieceCount() {return mPieceCount;}

  enum MoveStatus {NoMove,MoveStart,MoveEnd,PrepareNew};

  bool isRunning() {return mRunning;}
  void setRunning(bool b) {mRunning=b;}
  bool isIntro() {return mIntro;}
  void setIntro(bool b) {mIntro=b;}
  int moveStatus() {return mMoveStatus;}
  void setMoveStatus(int i) {mMoveStatus=i;}
  int moveSecondX() {return mMoveSecondX;}
  int moveSecondY() {return mMoveSecondY;}
  double speedModify() {return mSpeedModify;}

  void firstPiece(int x,int y);
  bool secondPiece(int x,int y);
  bool moveFinished();

  bool calcPath(int x0,int y0,int x1,int y1);

  void newGame();
  void endGame();
  int moveColour() {return (int)mMoveCol;}
  int moveSprite() {return (int)mMoveSprite;}
  ListData *waypoint();
  void removeWaypoint(ListData *waypoint);
  void removePiece(int x,int y);

  void redrawNewPieces();
  bool noRedraw();

  void changeTheme(QString path);

  signals:
  void gameOver(int);
  void statusMove(const QString &);



  protected:

  int findLevel(QString name);
  void deleteAllPieces();
  void getRandomPosition(int &x,int &y);
  bool insertNewPieces(int oldno,int newno,bool replace=false);
  int getRandomPiece();
  /**
   * recursive path filling
   */
  bool fillPath(int x0,int y0);
  void addList(int x,int y,int level,bool force=false);
  int calcMinNeighbour(int x, int y,int value);

  /**
   * callculates the index to the arrays
   */
  int aIndex(int x,int y) {return x+y*boardSize();}
   /**
    * inittilizses the random generator
    */
  void initRandom();

  bool gameOver();
  bool isJoker(int col);
  bool isSuperJoker(int col);
  int getLine(int x,int y,int &col,int dx,int dy,int remove,int &jokerlen);
  int calcLen(int len1,int len2,bool samecol,int &resultalign,int val1,int val2,int jlen1,int jlen2);
  int checkForFive2(int x,int y,int align);
  int checkForPiece(int &col,int x,int y,int &count,int align);
  int calcScore(unsigned int count,int direction);
  int calcNumberNewPieces();
  void removePieve(int x,int y);

  public slots:

  /**
   * Called by the sprite when the insert animation is finished
   */
  void slotPieceInserted(QCanvasItem *sprite,int mode);
  /**
   * Called by the sprite when the remove animation is finished
   */
  void slotPieceRemoved(QCanvasItem *item,int mode);

  /**
   * Called by the sprite when the remove animation is finished
   */
  void slotGameOverMove(QCanvasItem *item,int mode);

 	
  public:	
    /** the list of the views currently connected to the document */
    GameView *pView;	

  private:
    //QMap<QString,QString> mFilenameMap;
    //QMap<QString,QString> mMasknameMap;
    /** the modified flag of the current document */
    bool modified;
    QString title;
    QString absFilePath;
    KRandomSequence mRandom;

    QMemArray<int> mColour;     // Color occupation
    QMemArray<int> mNumber;     // Sprite numbers
    QMemArray<int> mNewNumber;     // Sprite numbers
    QMemArray<int> mPath;
    QMemArray<int> mBoard;
    QMemArray<int> mRemoved;  // Removed pieces
    QMemArray<int> mNew;
    int mNewPieces;         // so many new pieces on board
    int mScore;             // score

    bool mRunning;          // game running
    bool mIntro;            // intro running
    int mMoveStatus;        // movestatus
    int mMoveSprite;        // moving sprite

    int mMoveFirstX;        // Store move
    int mMoveFirstY;
    int mMoveSecondX;
    int mMoveSecondY;
    int mMoveCol;
    int mPieceCount;        // pieces on board



    MList mList;
    MList mMoveList;
    QValueList<int> mScoreList;  // scores
    QValueList<int> mPropList;  // Probability to draw piece
    int mMaxProp;               // sum of this list
    bool mNoPathFinding;   // do not find real path
    double mSpeedModify;   // multiplication of moce speed
    double mAnimModify;       // animation speed modify 

    bool mFirstTime;       // first time start?
    int mChangeBoardSize;
    int mChangeNewSize;
    int mWallpaper;
    bool mDecorations;
    int mMoves; // How many moves

    KConfig *mConfig;
    QWidget *mParent;

    QString mLevel;            // Level
    QPtrList<LevelData> mLevelList;
    LevelData mCurrentLevel;
};


#endif 
