/***************************************************************************
                          quintalignview.cpp  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qrect.h>
#include <qpoint.h>
#include <qmap.h>
#include <qstring.h>
#include <qprogressdialog.h>
#include <qwmatrix.h>
#include <qwidget.h>
#include <qimage.h>
#include <qbitmap.h>
#include <qpainter.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>
#include <math.h>
#include <stdio.h>


// application specific includes
#include "quintalignview.h"
#include "quintaligndoc.h"
#include "quintalign.h"
#include "preloaddlg.h"

#define MAX_COLOR    15  // 2^n-1
#define SHOW_ALL_PIECES  // show intro with all pieces

#define COL_BACKGROUND    QColor(0,0,128)
#define COL_STATUSBORDER black
#define COL_STATUSFIELD  QColor(130,130,255)
#define COL_STATUSDARK   QColor(0,0,65)
#define COL_STATUSLIGHT  QColor(210,210,255)
#define COL_PLAYER       QColor(255,255,0)

// Signals of the signal emitter:
// 1: move finished
// 2: animation finished



GameView::GameView(QString grafixdir,QWidget *parent, const char *name) : QCanvasView(0,parent, name)
{
  setVScrollBarMode(AlwaysOff);
  setHScrollBarMode(AlwaysOff);

  
  QDir dir(grafixdir);
  setGrafixDir(dir.absPath()+QString("/"));
  if (global_debug>0)
    kdDebug() << "path="<<grafixdir<< " --->"<< mGrafix<<endl;
  setBackgroundMode(PaletteBase);
  move(0,0);
  adjustSize();
  mCanvas=new QCanvas(parent);
  mCanvas->resize(parent->width(),parent->height());
  mCanvas->setDoubleBuffering(true);
  setCanvas(mCanvas);
 
  mConfig=0;
  initView();  
  mSpriteDict.clear();
  //mItemDict.setAutoDelete(true);
  mSpriteDict.setAutoDelete(true);
  mItemDict.clear();

  QPixmap background; // empty pixmap sets default
  setBackground(background);

}

void GameView::initView(bool deleteall)
{
  QPoint defaultoffset;

  QString file=mGrafix+QString("grafix.rc");
  if (global_debug>0)
    kdDebug() << "Init view with config="<<file<<endl;
  mConfig=new KConfig(file,false,false);
  if (!mConfig->hasGroup("game"))
  {
    kdDebug() << "ERROR: Could not find grafix data file " << file << endl;
  }
  mConfig->setGroup("game");
  int period=mConfig->readNumEntry("advanceperiod",25);
  mDecoration=mConfig->readListEntry("decoration");
  if (global_debug>10)
    kdDebug() << " intiView mDecorations=" << mDecoration.count() << endl;

  //  mCanvas->setUpdatePeriod(25);
  mCanvas->setAdvancePeriod(period);

  mScoreDigits=4;

  boardSize=pieceSize*getDocument()->boardSize();
  defaultoffset=QPoint(10,10);
  boardOffset=mConfig->readPointEntry("boardoffset",&defaultoffset);

  defaultoffset=QPoint(10,90);
  newOffset=mConfig->readPointEntry("newoffset",&defaultoffset)+boardOffset+QPoint(boardSize.width(),0);

  defaultoffset=QPoint(10,30);
  scoreOffset=mConfig->readPointEntry("scoreoffset",&defaultoffset)+boardOffset+QPoint(boardSize.width(),0);

  defaultoffset=QPoint(0,0);
  gameoverOffset=mConfig->readPointEntry("gameoveroffset",&defaultoffset);
  defaultoffset=QPoint(0,0);
  gameoverDestination=mConfig->readPointEntry("gameoverdestination",&defaultoffset);

  // Careful: from here we change the config group!! 
  // Get size of a QPixmap field
  QString name;
  QPixmap *sizePix;
  mConfig->setGroup("boardpiece");
  name=mConfig->readEntry("file");
  sizePix=loadPixmap(name);
  pieceSize=sizePix->size();
  delete sizePix;

  mConfig->setGroup("number");
  name=mConfig->readEntry("loadfile");
  name.sprintf(name.latin1(),0);
  sizePix=loadPixmap(name);
  numberSize=sizePix->size();
  delete sizePix;

  if (deleteall) deleteAllItems();
}

GameView::~GameView()
{
  if (mCanvas) delete mCanvas;
  if (mConfig) delete mConfig;
}
void GameView::setBackground(QPixmap &background)
{
  if (global_debug>10)
    kdDebug()  << "Setting background!"<<endl;
  setBackgroundColor(COL_BACKGROUND);
  if (!mCanvas) return ;
  mCanvas->setBackgroundColor(COL_BACKGROUND);
  if (background.isNull())
  {
    if (global_debug>10)
      kdDebug() << "Setting default background" << endl;
    mConfig->setGroup("game");
    QString name=mConfig->readEntry("defaultbackground");
    QPixmap *pm=loadPixmap(name);
    if (pm && !pm->isNull()) background=*pm;
  }
  if (!background.isNull())
  {
    mCanvas->setBackgroundPixmap(background);
  }
  else
  {
    kdDebug() << "WARNING: Background pixmap not defined" << endl;
  }
}

void GameView::changeTheme(QString s,bool deco)
{

  QString key;
  if (deco) drawDecoration(true);

  PSprite *sprite;
  QDict<QCanvasItem> mTempDict(mItemDict);


  // Remove all sprites..and free memory
  QDictIterator<QCanvasItem> it( mTempDict );
  while ( it.current() )
  {
    key=it.currentKey();
    sprite=(PSprite *)(mItemDict.take(key));
    if (sprite) delete sprite;
    ++it;
  }
  mSpriteDict.clear();

  setGrafixDir(s);
  initView(false);

  QPixmap background; // empty pixmap sets default
  setBackground(background);
}

void GameView::Debug()
{
  kdDebug() << "Canvassize " << mCanvas->width() << " " << mCanvas->height() << endl;
  kdDebug() << "Items on canvas " << mCanvas->allItems().count() << endl;
  kdDebug() << "Spritecache size=" << mSpriteDict.count() << endl;
  QCanvasItemList list=mCanvas->allItems();
  int cnt=0,acnt=0;
 for (QCanvasItemList::ConstIterator it=list.begin(); it!=list.end(); ++it)
 {
		QCanvasItem *emp=*it;
    kdDebug() << "Item("<<cnt<<") " << emp << " animated=" << emp->animated()
              << " show=" << emp->isVisible()  << " active=" << emp->isActive() << endl;
    if (emp->animated()) acnt++;
    cnt++;
  }
 kdDebug() << "Item:" << cnt << " animated=" << acnt << endl;

  QDictIterator<QCanvasItem> it( mItemDict ); 
  kdDebug()  << "Sprite items in cache=" << mItemDict.size() << endl;
  while ( it.current() )
  {
    //QCanvasItem *item=it.current();
    kdDebug() << it.currentKey() << endl;
    ++it;
  }
}

GameDoc *GameView::getDocument() const
{
  GameApp *theApp=(GameApp *) parentWidget();
  return theApp->getDocument();
}

void GameView::drawBoard(int size,int newsize,bool remove)
{
  int x,y;
  QPoint origin;
  QCanvasItem *sprite;

  for (y=0;y<size;y++)
  {
    for (x=0;x<size;x++)
    {
      
      if (remove)
      {
        deleteItem("boardpiece",x+size*y);
        sprite=0;
      }
      else sprite=getItem("boardpiece",x+size*y);
      origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
      if (sprite)
      {
        sprite->move(origin.x(),origin.y());
        sprite->show();
      }
    }
  }
  
  x=0;
  for (y=0;y<newsize;y++)
  {
    if (remove)
    {
      deleteItem("newpiece",y);
      sprite=0;
    }
    else sprite=getItem("newpiece",y);
    origin=newOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
    if (sprite)
    {
      sprite->move(origin.x(),origin.y());
      sprite->show();
    }
  }
}

QCanvasItem * GameView::drawPiece(int x,int y,char col,int no,int mode)
{
  QPoint origin;
  if (mode==1)
    origin=newOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
  else
    origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
  
  QString name;
  name=QString("piece%1").arg((int)col);
  QCanvasItem *sprite=getItem(name,no);
  if (sprite)
  {
    sprite->move(origin.x(),origin.y());
    sprite->show();
  }
  return sprite;
}


void GameView::drawDecoration(bool remove)
{
  PSprite *sprite;
  
  for ( QStringList::Iterator it = mDecoration.begin(); it != mDecoration.end(); ++it )
  {
    int center=0;
    mConfig->setGroup(*it);
    unsigned int align=mConfig->readNumEntry("align",0);
    QPoint origin;
    if (align==1) origin=boardOffset;
    else if (align==2) origin=boardOffset+QPoint(boardSize.width(),0);
    else if (align==3) origin=boardOffset+QPoint(0,boardSize.height());
    else if (align==4) origin=boardOffset+QPoint(boardSize.width(),boardSize.height());
    else if (align==5) {origin=boardOffset+QPoint(boardSize.width()/2,0);center=1;}
    else if (align==6) {origin=boardOffset+QPoint(boardSize.width(),boardSize.height()/2);center=2;}
    else if (align==7) {origin=boardOffset+QPoint(boardSize.width()/2,boardSize.height());center=3;}
    else if (align==8) {origin=boardOffset+QPoint(0,boardSize.height()/2);center=4;}
    else origin=QPoint(0,0);

    if (global_debug>10)
      kdDebug()  << "Decoration:"<< *it << " align=" << align << " center=" << center << " remove=" << remove <<  endl;

    if (remove)
    {
      deleteItem(*it);
      if (global_debug>20)
        kdDebug() << " deleteItem " << *it << endl;
      sprite=0;
    }
    else sprite=(PSprite *)getItem(*it);
    if (sprite)
    {
      if (center==1) sprite->move(origin.x()-sprite->width()/2,origin.y()-sprite->height()); // x center
      else if (center==2) sprite->move(origin.x(),origin.y()-sprite->height()/2); // y center
      else if (center==3) sprite->move(origin.x()-sprite->width()/2,origin.y()); // x center
      else if (center==4) sprite->move(origin.x()-sprite->width(),origin.y()-sprite->height()/2); // y center
      else sprite->move(origin.x(),origin.y());
      sprite->show();
      if (global_debug>20)
        kdDebug() << "sprite=("<<sprite->x()<<","<<sprite->y()<<endl;
    }
  }
     
}

QCanvasItem * GameView::drawGameOver(int /*sequence*/,bool movement,bool remove)
{
  PSprite *sprite;

  if (global_debug>10)
    kdDebug() << "Items on canvas " << mCanvas->allItems().count() << endl;

  if (remove)
  {
    deleteItem("gameover");
    sprite=0;
  }
  else sprite=(PSprite *)getItem("gameover");
  if (sprite)
  {
    int w=sprite->pixmap()->width();
    int dx=boardOffset.x()+(boardSize.width()-w)/2;
    if (movement)
    {
      sprite->move(dx+gameoverOffset.x(),gameoverOffset.y());
      sprite->moveTo(dx+gameoverDestination.x(),gameoverDestination.y());
    }
    else
    {
      sprite->move(dx+gameoverDestination.x(),gameoverDestination.y());
    }
    sprite->setAnimation(0);
    sprite->show();
  }
  return sprite;
}

void GameView::preloadGraphics()
{
  PSprite *sprite;
  mConfig->setGroup("game");
  bool preload=mConfig->readBoolEntry("preload",true);
  if (preload)
  {
    LoadDlg *dlg=new LoadDlg(this,"loaddlg");
    dlg->show();
    // pieces
    dlg->setText(i18n("Loading sprites..."));
    KApplication::kApplication()->processEvents();
    dlg->setValue(0);
    for (unsigned int i=1;i<8;i++)
    {
      QString s=QString("piece%1").arg(i);
      sprite=(PSprite *)getItem(s,999);
      deleteItem(s,999);
      dlg->setValue(i*10);
    }

    dlg->setText(i18n("Loading game over animation ..."));
    KApplication::kApplication()->processEvents();
    sprite=(PSprite *)getItem("gameover",999);
    deleteItem("gameover",999);
    dlg->setValue(90);

    dlg->setText(i18n("Loading score ..."));
    KApplication::kApplication()->processEvents();
    sprite=(PSprite *)getItem("number",999);
    deleteItem("number",999);
    dlg->setValue(100);

    // Done
    dlg->setText(i18n("Starting up..."));
    KApplication::kApplication()->processEvents();
    dlg->setValue(100);
    delete dlg;
  }
}
void GameView::drawIntro(bool /*remove*/)
{
  PSprite *sprite;

  preloadGraphics();

#ifdef SHOW_ALL_PIECES
  for (unsigned int i=1;i<=8;i++)
  {
    QString s=QString("piece%1").arg(i);
    sprite=(PSprite *)getItem(s,999);
    sprite->setAnimation(0);
    sprite->move(0,pieceSize.width()*(i-1));
    sprite->show();
  }
#endif

  mConfig->setGroup("game");
  double introspeed=mConfig->readDoubleNumEntry("introspeed",0);
  if (global_debug>10)
    kdDebug() << "Read introspeed=" << introspeed << endl;
  QString name=mConfig->readEntry("defaultintro");

  QPixmap *pixmap=loadPixmap(name);
  if (!pixmap || pixmap->isNull())
  {
    kdDebug() << "WARNING: No intropicture defined."<<endl;
    return ;
  }
  QImage img=pixmap->convertToImage();  // slow
  GameDoc *doc=getDocument();
  int no;

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QColor col(img.pixel(x,y));
      if (col.red()>127 && col.green()>127 && col.blue()>127) no=1;
      else if (col.red()>127 && col.green()>127 ) no=3;
      else if (col.red()>127 && col.blue()>127) no=7;
      else if (col.green()>127 && col.blue()>127) no=5;
      else if (col.red()>127) no=2;
      else if (col.green()>127) no=4;
      else if (col.blue()>127) no=6;
      else no=0;
      QString s=QString("piece%1").arg(no);
      if (no>0) 
      {
        int r0=doc->random(4);
        int x0=doc->random(100)-50;
        int y0=doc->random(100)-50;
        if (r0==1) // TR
        {
          x0+=contentsWidth();
        }
        else if (r0==2) // BL
        {
          y0+=height();
        }
        if (r0==3) // BR
        {
          x0+=contentsWidth();
          y0+=contentsHeight();
        }
        sprite=(PSprite *)getItem(s,x+y*img.width());
        sprite->setZ(x+y*img.width());
        sprite->move(x0,y0);
        sprite->moveTo(50+(pieceSize.width()/2)*x,(pieceSize.height()/2)*y);
        if (introspeed>0.0) sprite->setSpeed(introspeed);
        sprite->show();
        sprite->setAnimation(2,2.0);
      }
    }
  }

  if (global_debug>10)
    kdDebug() << "Items on canvas after intro " << mCanvas->allItems().count() << endl;


  QTimer::singleShot(7000,this,SLOT(slotContinueIntro()));
}

void GameView::slotContinueIntro()
{
  static unsigned int rcnt=0;
  static unsigned int scnt=0;
  if (!getDocument()->isIntro()) return ;

  QCanvasItemList list=mCanvas->allItems();

  int anim = 0;
  double mod=1.0;
  if (rcnt==0)
  {
    anim=1;
    mod=1.5;
    QTimer::singleShot(3000,this,SLOT(slotContinueIntro()));
    scnt=0;
  }
  else if (rcnt==1)
  {
    anim=4;
    QTimer::singleShot(70,this,SLOT(slotContinueIntro()));
  }
  else if (rcnt==2)
  {
    anim=0;
    QTimer::singleShot(3500,this,SLOT(slotContinueIntro()));
  }
  else if (rcnt==3)
  {
    anim=2;
    mod=1.5;
    QTimer::singleShot(7000,this,SLOT(slotContinueIntro()));
  }

  
  int cnt=0;
  for (QCanvasItemList::ConstIterator it=list.begin(); it!=list.end(); ++it)
  {
      PSprite *sprite=(PSprite *)(*it);

      if (anim==0)
      {
        sprite->hide();
      }
      else if (anim==4 && cnt==(int)scnt-5)
      {
        sprite->hide();
      }
      else if (anim==4 && cnt!=(int)scnt)
      {
      }
      else
      {
        if (!sprite->isVisible()) sprite->show();
        sprite->setAnimation(anim,mod);
      }

      cnt++;
  }

  if (anim==4)
  {
    scnt++;
    if (scnt==list.count()) rcnt++;
  }
  else
  {
    rcnt++;
  }
  if (rcnt>3) rcnt=0;

}

void GameView::drawScore(bool remove)
{
  int score=getDocument()->score();
  /*
  if (global_debug==42) score+=9990;
  if (global_debug==4242) score+=99990;
  if (global_debug==424242) score+=999990;
  */
  
  // Check how many digits
  int dgScore=1;
  int digits;
  int i;
  for (digits=0;digits<10;digits++)
  {
    if (score<dgScore) break;
    dgScore*=10;
  }
  //kdDebug() << " score=" << score << " dgScore=" << dgScore << " digits=" << digits << endl;
  if (digits>mScoreDigits)
  {
    if (global_debug>10)
      kdDebug() << "We need more digits" << endl;
    mScoreDigits=digits;
  }

  char no[mScoreDigits];
  for (i=0;i<mScoreDigits;i++)
  {
    no[mScoreDigits-i-1]=score%10;
    score/=10;
    //kdDebug() <<"["<<i<<"]="<<no[mScoreDigits-i-1]<<endl; 
  }

  //QPoint nOffset=QPoint((scoreSize.width()-4*numberSize.width())/2,(scoreSize.height()-numberSize.height())/2);
  QPoint borigin;
  QPoint sorigin;
  QPoint offset;

  sorigin=scoreOffset;
  borigin=scoreOffset;

  PSprite *numberSprite;
  PSprite *backSprite;
  for (int i=0;i<mScoreDigits;i++)
  {

    QString b;
    if (i==0) b=QString("score_left");
    else if (i==mScoreDigits-1) b=QString("score_right");
    else b=QString("score_middle");
    QString s=QString("number");
    if (remove)
    {
      deleteItem(s,i);
      numberSprite=0;
      if (i==mScoreDigits-1) deleteItem(b,0);
      else deleteItem(b,i);
      backSprite=0;
    }
    else
    {
      numberSprite=(PSprite *)getItem(s,i);
      if (i==mScoreDigits-1) backSprite=(PSprite *)getItem(b,0);
      else backSprite=(PSprite *)getItem(b,i);
    }

    if (backSprite && numberSprite)
    {
      backSprite->move(borigin.x(),borigin.y());
      backSprite->show();

      offset=QPoint((backSprite->width()-numberSprite->width())/2,(backSprite->height()-numberSprite->height())/2);
      
      numberSprite->setFrame(no[i]);
      numberSprite->move(sorigin.x()+offset.x(),sorigin.y()+offset.y());
      numberSprite->show();

      sorigin+=QPoint(numberSprite->width(),0);
      borigin+=QPoint(backSprite->width(),0);
    }
  }
}

// paint function
void GameView::Paint(QPainter * /*p*/)
{
}

void GameView::resizeEvent(QResizeEvent *e)
{
  if (mCanvas) mCanvas->resize(e->size().width(),e->size().height()); 
}

// paint event
void GameView::paintEvent( QPaintEvent * /*e*/)
{
#ifdef NOCANVAS
  {
    QPainter paint( this );
    paint.setClipRect(e->rect());
    Paint( &paint );			
  }
#endif
}


// mouse click event
void GameView::contentsMousePressEvent( QMouseEvent *mouse )
{
  bool cheat=false;
  if (mouse->button()==MidButton) cheat=true;
  if (!cheat && mouse->button()!=LeftButton) return ;
  QPoint point;
  point=mouse->pos();
  point-=boardOffset;
  if (point.x()<0 || point.y()<0) return ; 
  if (point.x()>=boardSize.width() || point.y()>=boardSize.height()) return ;


  int movex,movey;
  movex=point.x()/pieceSize.width();
  movey=point.y()/pieceSize.width();
//  kdDebug() << "Move to " << movex << " , " << movey << endl;

  GameDoc *doc=getDocument();
  if (!doc->isRunning()) 
  {
    if (global_debug>10)
      kdDebug() << "Game not running"<<endl;
    return ;
  }

  // Move all to gamedoc!!!!
  if (doc->moveStatus()==GameDoc::MoveEnd) return ; // new move not yet possible
  else if (doc->moveStatus()==GameDoc::NoMove) 
  {
    if (doc->getBoard(movex,movey)!=0)
    {
      if (cheat) doc->removePiece(movex,movey);
      else doc->firstPiece(movex,movey);
    }
    else if (global_debug>10)
      kdDebug() << "No piece here" <<endl;

  }
  else if (doc->moveStatus()==GameDoc::MoveStart) 
  {
    // TODO Check for move possible
    if (doc->getBoard(movex,movey)!=0)
    {
      // Perform first move
      doc->firstPiece(movex,movey);
    }
    else
    {
      if (!doc->secondPiece(movex,movey))
      {
        if (global_debug>10)
          kdDebug() << "No path found" << endl;
      }
      else
      {
        QString s=QString("piece%1").arg(doc->moveColour());
        PSprite *sprite=(PSprite *)(getItem(s,doc->moveSprite()));
        sprite->setAnimation(0);
        connect(sprite->emitter(),SIGNAL(signalNotify(QCanvasItem *,int)),
                this,SLOT(slotMoveTo(QCanvasItem *,int)));
        slotMoveTo(sprite,1);
      }
    }
  }

}
void GameView::slotMoveTo(QCanvasItem *sprite,int mode)
{
  if (!sprite) return ;
  if (mode!=1)
  {
    if (global_debug>10)
      kdDebug() << "Wrong mode in slotMoveTo -> returning "<<endl;
    return ;
  }
  GameDoc *doc=getDocument();

  int x=0,y=0;
  ListData *waypoint=doc->waypoint();
  if (waypoint) 
  {
    x=waypoint->x;
    y=waypoint->y;
  }

  if (!waypoint)
  {
    doc->moveFinished();
    QObject *ob=((PSprite *)sprite)->emitter(false);
    if (ob) disconnect(ob,SIGNAL(signalNotify(QCanvasItem *,int)),
                       this,SLOT(slotMoveTo(QCanvasItem *,int)));
    ((PSprite *)sprite)->deleteEmitter();
  }
  else
  {
    // kdDebug() << " view: going to waypoint=("<<x<<","<<y<<")"<<endl;
    QPoint origin;
    origin=boardOffset+QPoint(x*pieceSize.width(),y*pieceSize.height());
    ((PSprite *)sprite)->moveTo(origin.x(),origin.y());
  }
  doc->removeWaypoint(waypoint);
}


void GameView::updateStatus()
{
  /*
  update(status_rect1);
  update(status_rect2);
  */
}

QPixmap * GameView::loadPixmap(QString file,QString mask)
{
  QPixmap *newP=new QPixmap;
  bool result1=false;
  bool result2=false;
  if (!file.isNull())
  {
    result1=newP->load(mGrafix+file);
  }
  if (result1 && !mask.isNull())
  {
    QBitmap bitmask;
    if (mask==QString("auto"))  // self mask..slow but quick to create
    {
      newP->setMask( newP->createHeuristicMask() );
      result2=true;
    }
    else
    {
      result2=bitmask.load(mGrafix+mask);
      if (result2) newP->setMask(bitmask);
    }
  }
  if (global_debug>100)
    kdDebug() << "GameView::loadPixmap: file="<<file<<"   mask="<<mask<<"   result1="<<result1<<"   result2="<<result2<<endl;
  return newP;
}

void GameView::deleteAllItems()
{
  QDictIterator<QCanvasItem> it( mItemDict ); 
  //kdDebug()  << "GameView::deleteAllItems items in cache=" << mItemDict.size() << endl;
  while ( it.current() )
  {
    QCanvasItem *item=it.current();
    mItemDict.remove(it.currentKey());
    delete item;
  }
}
void GameView::deleteItem(QString s,unsigned int no)
{
  QCanvasItem *item;
  QString name=s+QString("_%1").arg(no);
  //kdDebug() << "GameView::deleteItem name=" << name << endl;
  item=mItemDict[name];
  if (item)
  {
    mItemDict.remove(name);
//    kdDebug() << "deleteitem "<<name<<" as sprite="<<item<<endl;
    delete item;
  }
}

void GameView::deleteItem(QCanvasItem *item)
{
  QDictIterator<QCanvasItem> it( mItemDict ); 
  while ( it.current() )
  {
    if (item==it.current())
    {
 //     kdDebug() << "deleteitem sprite="<<item<<" it="<<it.currentKey()<<endl;
      mItemDict.remove(it.currentKey());
      delete item;
      return ;
    }
    ++it;
  }
} 

QCanvasItem * GameView::getItem(QString s,unsigned int no)
{
  QCanvasItem *item;
  QCanvasPixmapArray *pixmaparray;
  double z,movespeed;
  char anima[MAX_ANIM];
  char animb[MAX_ANIM];
  char animdir[MAX_ANIM];
  char animspeed[MAX_ANIM];
  unsigned int i;

  if (global_debug>100)
    kdDebug() << " -> getItem("<<s<<","<<no<<")"<<endl;
  item=mItemDict[s+QString("_%1").arg(no)];
  if (item)
  {
    if (global_debug>100)
      kdDebug() << "found item "<<s+QString("_%1").arg(no)<<" as directly existing "<<item << endl;
    return item;
  }

  // check whether we can copy a sprite
  SpriteCache *spriteCache=mSpriteDict[s];
  if (!spriteCache)
  {
    // Now first time load the items
    if (!mConfig->hasGroup(s))
    {
      kdDebug() << "Item "<<s <<" not defined !!! " <<endl;
      return 0;
    }
    if (global_debug>10)
      kdDebug() << " +++++ Creating new item "<<s << " count in list="<<mItemDict.count()<< " no="<<no <<  endl;
    mConfig->setGroup(s);
    movespeed=mConfig->readDoubleNumEntry("movespeed",10.0);
    QPoint defaultoffset=QPoint(0,0);
    QPoint offset=mConfig->readPointEntry("offset",&defaultoffset);
    QValueList<int> filterList;
    filterList=mConfig->readIntListEntry("colorfilter");
    QValueList<int> transformList;
    transformList=mConfig->readIntListEntry("transformfilter");
    QStringList operationList=mConfig->readListEntry("pixmaps");
    z=mConfig->readDoubleNumEntry("z",0.0);
    // Append default entryy
    if (operationList.count()==0)
    {
      QString s;
      operationList.append(s);
    }


    QValueList<int> animList;
    // frame A, frame B, mode, delay
    for (i=0;i<MAX_ANIM;i++)
    {
      QString animname=QString("anim%1").arg(i);
      animList=mConfig->readIntListEntry(animname);
      if (mConfig->hasKey(animname) && animList.count()>=4)
      {
        anima[i]=(char)animList[0];
        animb[i]=(char)animList[1];
        animdir[i]=(char)animList[2];
        animspeed[i]=(char)animList[3];
      }
      else
      {
        anima[i]=0;
        animb[i]=0;
        animdir[i]=0;
        animspeed[i]=0;
      }
      if (global_debug>100)
        kdDebug() << "Anim parameter loaded a="<<anima[i]<<" b="<<animb[i]<<" dir="<<animdir[i]<<" speed="<<animspeed[i]<<endl;
    }
    
    QPixmap *pixmap;
    QPtrList<QPixmap> pixlist;
    pixlist.setAutoDelete(true);
    QPtrList<QPoint> hotlist;
    hotlist.setAutoDelete(true);

    for ( QStringList::Iterator it = operationList.begin(); it !=operationList.end(); ++it )
    {
      QString name=*it;
      QString type=mConfig->readEntry(name+"type");
      if (type.isNull()) type=QString("load"); // default load
      //kdDebug() << " Processing operation " << (name.isNull()?"default":name) << "type="<<type << endl;
      unsigned int number=mConfig->readNumEntry(name+"number",1);
      // Load a given set of images or replace a %d by a sequence if there are 
      // less image names than number given
      if (type==QString("load"))
      {
        QStringList pixfile=mConfig->readListEntry(name+"file");
        QStringList maskfile=mConfig->readListEntry(name+"mask");
        // Read images
        for (i=0;i<number;i++)
        {
          QString tmpfile,tmpmask;
          if (pixfile.count()>=number || pixfile.count()>i+1) tmpfile=pixfile[i];
          else tmpfile.sprintf(pixfile.last().latin1(),i);
          if (maskfile.count()>=number || maskfile.count()>i+1) tmpmask=maskfile[i];
          else if (maskfile.count()>0) tmpmask.sprintf(maskfile.last().latin1(),i);
          
          pixmap=loadPixmap(tmpfile,tmpmask);
          if (!pixmap) kdError() << "Could not create pixmap="<<tmpfile << " with mask " << tmpmask << endl;
          else
          {
            if (transformList.count()>0)
            {
              if (transformList[0]==1 && transformList.count()==2)  // rotate
              {
                QWMatrix rotate;
                rotate.rotate(transformList[1]);
                *pixmap=pixmap->xForm(rotate);
              }
              else if (transformList[0]==2 && transformList.count()==3) // scale
              {
                QWMatrix scale;
                scale.scale((double)transformList[1]/100.0,(double)transformList[2]/100.0);
                *pixmap=pixmap->xForm(scale);
              }
            }
            // apply colorfilter
            if (filterList.count()>0)
            {
              // Only filter 1 HSV implemented
              if (filterList[0]==1 && filterList.count()==4) changeHSV(pixmap,filterList[1],filterList[2],filterList[3]);
              else if (filterList[0]==2 && filterList.count()==2) changeGrey(pixmap,filterList[1]);
              else if (filterList[0]==2 && filterList.count()==1) changeGrey(pixmap);
              else kdDebug() << "WARNING: Colorfilter parameter incorrect "<< endl;
            }
            pixlist.append(pixmap);
            QPoint *copyoffset=new QPoint(-offset);
            hotlist.append(copyoffset);
          }
        }
      }
      // Scale some images in given axis
      else if (type==QString("scale"))
      {
        // scale images
        QString tmpfile=mConfig->readEntry(name+"file");
        QString tmpmask=mConfig->readEntry(name+"mask");
        int axis=mConfig->readNumEntry(name+"axis",1);
        int maxpiece=mConfig->readNumEntry(name+"max",number);
        //kdDebug() << " Scaling " << number << " pics axis="<<axis<<" max="<<maxpiece<<" file="<<tmpfile<< " mask="<<tmpmask<<endl;

        pixmap=loadPixmap(tmpfile,tmpmask);
        for (unsigned int j=0;j<(unsigned int)number;j++)
        {
          QWMatrix matrix;
          double sc=(double)(maxpiece-j)/(double)(maxpiece);
          if (axis==1) matrix.scale(sc,1.0);
          else if (axis==2) matrix.scale(1.0,sc);
          else matrix.scale(sc,sc);
          QPixmap *copypixmap=new QPixmap(pixmap->xForm(matrix));
          if (transformList.count()>0)
          {
            if (transformList[0]==1 && transformList.count()==2)  // rotate
            {
              QWMatrix rotate;
              rotate.rotate(transformList[1]);
              *copypixmap=copypixmap->xForm(rotate);
            }
            else if (transformList[0]==2 && transformList.count()==3) // scale
            {
              QWMatrix scale;
              scale.scale((double)transformList[1]/100.0,(double)transformList[2]/100.0);
              *copypixmap=copypixmap->xForm(scale);
            }
          }
          // apply color filter
          if (filterList.count()>0)
          {
            // Only filter 1 HSV implemented
            if (filterList[0]==1 && filterList.count()==4) changeHSV(copypixmap,filterList[1],filterList[2],filterList[3]);
            else if (filterList[0]==2 && filterList.count()==2) changeGrey(copypixmap,filterList[1]);
            else if (filterList[0]==2 && filterList.count()==1) changeGrey(copypixmap);
            else kdDebug() << "WARNING: Colorfilter parameter incorrect "<< endl;
          }
          pixlist.append(copypixmap);
          QPoint *copyoffset=new QPoint((-pixmap->width()+copypixmap->width())/2,(-pixmap->height()+copypixmap->height())/2);
          hotlist.append(copyoffset);
        }
        delete pixmap;

      }
      else
      {
        kdDebug() << "WARNING: Unknown algorithm " << type << " for " << name << " not supported " << endl;
      }
    }// end create images


    //kdDebug() <<"Pixarray count="<<pixlist.count()<<endl;
    if (pixlist.count()<1) return 0;

    pixmaparray=new QCanvasPixmapArray(pixlist,hotlist);
    if (!pixmaparray) kdError() << "Could not create pixmaparray" << endl;
    spriteCache=new SpriteCache;
    spriteCache->pixmapArray=pixmaparray;
    spriteCache->z=z;
    spriteCache->mMoveSpeed=movespeed;
    for (i=0;i<MAX_ANIM;i++)
    {
      spriteCache->anima[i]=anima[i];
      spriteCache->animb[i]=animb[i];
      spriteCache->animdir[i]=animdir[i];
      spriteCache->animspeed[i]=animspeed[i];
    }
    mSpriteDict.insert(s,spriteCache);
  }
  else // retrieve fomr buffers
  {
    pixmaparray=spriteCache->pixmapArray;
    z=spriteCache->z;
    movespeed=spriteCache->mMoveSpeed;
    for (i=0;i<MAX_ANIM;i++)
    {
      anima[i]=spriteCache->anima[i];
      animb[i]=spriteCache->animb[i];
      animdir[i]=spriteCache->animdir[i];
      animspeed[i]=spriteCache->animspeed[i];
    }
    //kdDebug() << "Sprite "<<s<<" was cached. copying to " << no << endl;
  }
    
  PSprite *sprite=new PSprite(pixmaparray,mCanvas);
  sprite->setSpeed(movespeed*getDocument()->speedModify());
  sprite->setZ(z);
  for (i=0;i<MAX_ANIM;i++)
  {
    sprite->prepareAnimation(i,anima[i],animb[i],animdir[i],animspeed[i]);
  }
  sprite->setAnimation(0);
  item=sprite;
  if (global_debug>100)
    kdDebug() << "sprite"<<no<<" as '"<<s+QString("_%1").arg(no)<<"' inserted  (z="<<z<<") ok  and dictionary inserted ok" << endl;
  mItemDict.insert(s+QString("_%1").arg(no),item);
  return item;
}

void GameView::changeHSV(QPixmap *pixmap,int dh,int ds,int dv)
{
  if (!pixmap || (dh==0 && ds==0 && dv==0)) return ;
  if (pixmap->isNull()) return ;
  if (pixmap->width()==0 && pixmap->height()==0) return ;
  
  int h,s,v;
  QColor black=QColor(0,0,0);
  QImage img=pixmap->convertToImage();  // slow

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QRgb pix=img.pixel(x,y);
      QColor col(pix);
      col.hsv(&h,&s,&v);
      if (col==black) continue;  // speed up?
      h=((unsigned int)(h+dh))%360;
      s=((unsigned int)(s+ds)%256);
      v=((unsigned int)(v+dv)%256);
      col.setHsv(h,s,v);
      img.setPixel(x,y,qRgba(col.red(),col.green(),col.blue(),qAlpha(pix)));
    }
  }
  pixmap->convertFromImage(img); // slow
}
void GameView::changeGrey(QPixmap *pixmap,int lighter)
{
  if (!pixmap) return ;
  if (pixmap->isNull()) return ;
  if (pixmap->width()==0 && pixmap->height()==0) return ;
  
  QImage img=pixmap->convertToImage();  // slow

  for (int y=0;y<img.height();y++)
  {
    for (int x=0;x<img.width();x++)
    {
      QRgb pix=img.pixel(x,y);
      int gray=qGray(qRed(pix),qGreen(pix),qBlue(pix));
      QColor col(gray,gray,gray);
      if (lighter>0) col=col.light(lighter);
      if (lighter<0) col=col.dark(-lighter);
      img.setPixel(x,y,qRgba(col.red(),col.green(),col.blue(),qAlpha(pix)));
    }
  }
  pixmap->convertFromImage(img); // slow
}

        

// ---------------- Sprite ------------
PSprite::PSprite(QCanvasPixmapArray *p,QCanvas *c) : QCanvasSprite(p,c)
{
  for (unsigned int i=0;i<MAX_ANIM;i++)
  {
    animb[i]=0;
    anima[i]=0;
    animdir[i]=0;
    animspeed[i]=0;
  }
  mAnimModify=1.0;
  animationNumber=0;
  animSpeedCnt=0;
  currentAnimDir=0;
  mEmitter=0;
  targetX=0;
  targetY=0;
  moveStatus=0;
  speed=1.0;
  mData=0;
}

PSprite::~PSprite()
{
  if (mEmitter) delete mEmitter;
}

int PSprite::width() 
{
  //kdDebug() << "Width:"<< QCanvasSprite::width()<<endl;
  return QCanvasSprite::width();
}
int PSprite::height() 
{
  //kdDebug() << "height:"<< QCanvasSprite::height()<<endl;
  return QCanvasSprite::height();
}

void PSprite::advance(int stage)
{
  //kdDebug()  << "PSprite::Advance beginmove status of " << this << " = " << moveStatus << " stage=" << stage << endl;
  if (stage!=1) return ;
  if (!isVisible()) return ;

  bool isAnim=true;
  bool isMoving=false;

  int emitsignal=0;
//  kdDebug() << "Advance for " << this << endl;

  // Animation
  // mode  0: no animation
  //       1: single shot a->b
  //      -1: single shot b->a
  //       2: cycle  a->b->a
  //      -2: cycle  b->a->b
  //       3: cycle  a->b 
  //      -3: cycle  b->a 
  animSpeedCnt++;
  if (animspeed[animationNumber]==0)
  {
    // nothign to do?
    //setFrame(anima[animationNumber]);
    isAnim=false;
    animSpeedCnt=0;
  }
  if (animSpeedCnt>=(int)(double)animspeed[animationNumber]*mAnimModify)
  {
    switch(animdir[animationNumber])
    {
      case 1: 
        if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
        else emitsignal=2;
      break;
      case -1: 
        if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
        else emitsignal=2;
      break;
      case 2: 
        if (animdir[animationNumber]==currentAnimDir) 
        {
          if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
          else 
          {
            currentAnimDir=-currentAnimDir;
            if (frame()>0) setFrame(frame()-1);
          }
        }
        else
        {
          if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
          else 
          {
            currentAnimDir=-currentAnimDir;
            if (frame()+1<frameCount()) setFrame(frame()+1);
          }
        }
      break;
      case -2: 
        if (animdir[animationNumber]==currentAnimDir) 
        {
          if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
          else 
          {
            currentAnimDir=-currentAnimDir;
            if (frame()+1<frameCount()) setFrame(frame()+1);
          }
        }
        else
        {
          if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
          else 
          {
            currentAnimDir=-currentAnimDir;
            if (frame()>0) setFrame(frame()-1);
          }
        }
      break;
      case 3: 
        if (frame()+1 <= animb[animationNumber]) setFrame(frame()+1);
        else setFrame(anima[animationNumber]);
      break;
      case -3: 
        if (frame()-1 >= anima[animationNumber]) setFrame(frame()-1);
        else setFrame(animb[animationNumber]);
      break;
      default: //0
        isAnim=false;
        setFrame(0);
    }
    if (emitsignal) isAnim=false;
    
    animSpeedCnt=0;
  }

  // Move
  if (moveStatus)
  {
    double dx,dy;
    double vx,vy;
    dx=targetX-x();
    dy=targetY-y();
    // pure x,y movements
    vx=0.0;
    vy=0.0;
    if (fabs(dy)<=0.0001)
    {
      if (dx>0.0) vx=speed;
      else if (dx<0.0) vx=-speed;
      else vx=0.0;
    }
    else if (fabs(dx)<=0.0001)
    {
      if (dy>0.0) vy=speed;
      else if (dy<0.0) vy=-speed;
      else vy=0.0;
    }
    else // diagonal
    {
      double alpha=atan2(dy,dx);
      vx=cos(alpha)*speed;
      vy=sin(alpha)*speed;
    }

    if (fabs(dx)<=fabs(vx) && fabs(dy)<=fabs(vy))
    {
      move(targetX,targetY);
      moveStatus=0;
     // if (mEmitter) mEmitter->emitSignal(this,1);
     // else kdDebug() << "Error: emitter not defined" <<endl;
      emitsignal=1;
    }
    else if (fabs(dx)<=fabs(vx))
    {
      moveBy(dx,vy);
      isMoving=true;
    }
    else if (fabs(dy)<=fabs(vy))
    {
      moveBy(vx,dy);
      isMoving=true;
    }
    else
    {
      moveBy(vx,vy);
      isMoving=true;
    }
  }

  if (!isAnim && !isMoving)
  {
    setAnimated(false);
  }

  if (mEmitter && emitsignal)
  {
    //kdDebug() << " ADVANCE emits signal " << emitsignal << " for item "<< this << endl;
    mEmitter->emitSignal(this,emitsignal);
  }
}
void PSprite::setAnimation(int no,double animmodify)
{

  if (no<0||no>=MAX_ANIM )
  {
    kdDebug() << "ERROR: Animation number "<<no << " not supported " << endl;
    return ;
  }
  //if (no==animationNumber) return ;
  mAnimModify=animmodify;
  animationNumber=no;
  animSpeedCnt=0;
  char direction=animdir[no];
  currentAnimDir=direction;
  if (direction>=0)
  {
    setFrame(anima[no]);
  }
  else if (direction<0) setFrame(animb[no]);
  if (direction!=0 && animb[no]>=anima[no]) setAnimated(true);
  else setAnimated(false);
//  kdDebug() << this << " setAnimation("<<no<<") delay="<<animspeed[no]*animmodify<<" frames="<<anima[no]<<"->"<<animb[no]<<" mode="<<animdir[no]<<" animated="<<animated()<<endl;
}
void PSprite::prepareAnimation(int no,char a,char b, char mode, char speed)
{
  if (no<0 || no>=MAX_ANIM)
  {
    kdDebug() << "ERROR: Animation number " << no << " not supported!"<<endl;
    return ;
  }
  if (a>b)
  {
    kdDebug() << "ERROR: Animation start frame " << a << " bigger than end frame "<<b<<endl;
    return ;
  }
  if (a<0) a=0;
  if (b>=frameCount())
  {
    kdDebug() << "WARNING: prepareAnimation("<<no<<","<<a<<","<<b<<" over frameCount()"<<frameCount()<<"!!!"<<endl;
    b=frameCount()-1;
  }
  anima[no]=a;
  animb[no]=b;
  animdir[no]=mode;
  animspeed[no]=speed;
//  kdDebug() <<" prepareAnim: a="<<a<<" b="<<b<< " dir=" <<mode<<" speed="<<speed<<endl;
}

QObject *PSprite::emitter(bool createnew)
{
  if (!mEmitter)
  {
    if (createnew)
    {
      mEmitter=new SpriteNotify;
      //kdDebug() << " new emitter" << mEmitter<< " for sprite " << this << endl;
    }
  }
  else
  {
    mEmitter->refCnt++;
    //kdDebug() << " emitter" << mEmitter << " refcnt=" << mEmitter->refCnt<< " for sprite " << this << endl;
  }
  return (QObject *)mEmitter;
}

void PSprite::deleteEmitter()
{
  //kdDebug() << " delete emitter" << mEmitter << " refcnt=" << mEmitter->refCnt<< " for sprite " << this << endl;
  if (mEmitter && mEmitter->refCnt<=1)
  {
    delete mEmitter;
    mEmitter=0;
  }
  else mEmitter->refCnt--;
}

void PSprite::moveTo(int x,int y)
{
//  kdDebug()  <<"PSprite("<<this<<")::moveTo" << x << ", " << y << endl;
  targetX=x;
  targetY=y;
  moveStatus=1;
  setAnimated(true);
}

QCanvasPixmap* PSprite::pixmap() const
{
  return image();
}

SpriteCache::SpriteCache()
{
//  kdDebug() << " creating spritecache " << this << endl; 
  pixmapArray=0;
}
SpriteCache::~SpriteCache()
{
  if (pixmapArray) delete pixmapArray;
  pixmapArray=0;
//  kdDebug() << " deleting spritecache " << this << endl; 
}

