/***************************************************************************
                          quintalignview.h  -  description
                             -------------------
    begin                : 26 April 2001
    copyright            : (C) 2000 by Martin Heni
    email                : martin@heni-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUINTALIGNVIEW_H
#define QUINTALIGNVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qpixmap.h>
#include <qrect.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qcanvas.h>
#include <qdict.h>
#include <kconfig.h>
#include <kapp.h>

#include "quintalign.h"

#define MAX_ANIM  6


class GameDoc;

class SpriteNotify : public QObject
{
  Q_OBJECT
  public:
    SpriteNotify() :QObject(0,0) {refCnt=0;}
    void emitSignal(QCanvasSprite *parent,int mode) {emit signalNotify(parent,mode);}
    int refCnt;
  signals:
    void signalNotify(QCanvasItem *,int);
};

class PSprite: public QCanvasSprite
{
  public:
    PSprite(QCanvasPixmapArray *p,QCanvas *c);
    ~PSprite();
    void advance(int stage);
    void prepareAnimation(int no,char a,char b,char dir,char speed);
    /**
     *  choose animation 0,1,2
     *  animation goes from anima[no]..animb[no]
     *  direction=1: cycle a->b
     *           =-1: cycle b->a
     *           =2  singleshot a->b
     *           =-2 singleshot b->a
     *           =0  no animation
     */
    void setAnimation(int no,double animmodify=1.0);
    int animation() {return animationNumber;}
    double animationModify() {return mAnimModify;}
    QObject *emitter(bool createnew=true);
    void deleteEmitter();
    void moveTo(int x,int y);
    void setSpeed(double s) {speed=s;}
    void setData(int i) {mData=i;}
    int data() {return mData;}
    int width();
    int height();
    QCanvasPixmap* pixmap() const;


  private:
    SpriteNotify *mEmitter;
    char anima[MAX_ANIM]; 
    char animb[MAX_ANIM];
    char animdir[MAX_ANIM];
    char animspeed[MAX_ANIM];
    char animSpeedCnt;
    int animationNumber;
    int currentAnimDir;
    double targetX,targetY;
    double speed;
    int moveStatus;
    int mData;
    double mAnimModify;
    double mMoveSpeed;  

};

class SpriteCache 
{
  public:
    SpriteCache();
    ~SpriteCache();
    QCanvasPixmapArray *pixmapArray;
    double z;
    char anima[MAX_ANIM];   
    char animb[MAX_ANIM];   
    char animdir[MAX_ANIM];   
    char animspeed[MAX_ANIM];   
    double mMoveSpeed;  
};


/** The GameView class provides the view widget for the GameApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As GameView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the GameApp class to manipulate and display
 * the document structure provided by the GameDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class GameView : public QCanvasView
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    GameView(QString grafixdir,QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~GameView();

    /**
     * Initialises the view (done in the construcor)
     */
    void initView(bool deleteall=true);
    void setGrafixDir(QString dir) {mGrafix=dir;}

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a GameApp instance as a parent
     * widget to get to the window document pointer by calling the GameApp::getDocument() method.
     *
     * @see GameApp#getDocument
     */
    GameDoc *getDocument() const;

    void paintEvent( QPaintEvent * p);
    void Paint(QPainter *p);
    void updateStatus();
  void setBackground(QPixmap &background);

  QCanvasItem * drawGameOver(int sequence=0,bool movement=true,bool remove=false);
  void drawDecoration(bool remove=false);
  void drawIntro(bool remove=false);
  void drawScore(bool remove=false);
  void drawBoard(int size,int newsize,bool remove=false);
  QCanvasItem *drawPiece(int x,int y,char col,int no,int mode=0);
  QCanvasItem *getItem(QString s,unsigned int no=0);
  void deleteItem(QString s,unsigned int no=0);
  void deleteItem(QCanvasItem *item);
  void deleteAllItems();
  void preloadGraphics();
  void Debug();
  void changeTheme(QString path,bool deco);


  protected:
  void changeHSV(QPixmap *pixmap,int dh=0,int ds=0,int dv=0);
  void changeGrey(QPixmap *pixmap,int lighter=100);
  void resizeEvent(QResizeEvent *e);
  void contentsMousePressEvent ( QMouseEvent *m );


  QPixmap *loadPixmap(QString filename,QString maskname=0);
	
  protected slots:
  /**
   * moving stone timer
   */
    void slotMoveTo(QCanvasItem *,int);
    void slotContinueIntro();



  private:
    QSize pieceSize;    // Size of a board piece
    QSize boardSize;    // Size of a board 
    QSize numberSize;   // Size of a scorenumber 
    QPoint boardOffset; // Top/Left of board
    QPoint scoreOffset; // Top/Left of score
    QPoint gameoverOffset; // Top/Left of game over animation
    QPoint gameoverDestination; // Top/Left of game over animation target
    QPoint newOffset;   // Top/Left of new pieces

    QCanvas *mCanvas;    // our drawing canvas
    QDict<QCanvasItem> mItemDict;  // Spritename lookup
    QDict<SpriteCache> mSpriteDict;  // Spriteproperty lookup

    QString mGrafix;    // path to grafix dir
    KConfig *mConfig;   // configuration file

    QStringList mDecoration;  // List of decorations
    int mScoreDigits;   // how many score digits to show
	
};

#endif 
