#include <kdialogbase.h>
#include <qobject.h>
#include <qpushbutton.h>
#include "userlevel.h"
#include "userlevelbase.h"
#include "configdlg.h"
#include "leveldata.h"

/* 
 *  Constructs a UserlevelDlg which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
UserlevelDlg::UserlevelDlg( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : UserlevelDlgBase( parent, name, modal, fl )
{
  connect(PushButton1,SIGNAL(clicked()),this,SLOT(slotCreateNew()));
  connect(PushButton2,SIGNAL(clicked()),this,SLOT(slotEdit()));
  connect(PushButton3,SIGNAL(clicked()),this,SLOT(slotDelete()));
  connect(ListBox1,SIGNAL(highlighted(int)),this,SLOT(slotHighlighted(int)));
  slotHighlighted(-1);

}

/*  
 *  Destroys the object and frees any allocated resources
 */
UserlevelDlg::~UserlevelDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

void UserlevelDlg::slotHighlighted(int i)
{
  if (i>=0)
  {
    PushButton2->setEnabled(true);
    // TODO enable when delete works
    //    PushButton3->setEnabled(true);
  }
  else
  {
    PushButton2->setEnabled(false);
    PushButton3->setEnabled(false);
  }
}

void UserlevelDlg::slotCreateNew()
{
  LevelData *data=new LevelData;
  ConfigDlg *dlg=new ConfigDlg(this,"leveldlg");
  dlg->setData(data,true);
  if (dlg->exec()==QDialog::Accepted)
  {
    dlg->getData(data);
    data->save(mConfig);
    ListBox1->insertItem(data->name,0);
    mList.append(data->savename);
  }
  delete dlg;
}
void UserlevelDlg::slotEdit()
{
  LevelData *data=new LevelData;
  ConfigDlg *dlg=new ConfigDlg(this,"leveldlg");
  QString name=ListBox1->currentText();
  
  data->load(name,mConfig);
  dlg->setData(data,false);
  if (dlg->exec()==QDialog::Accepted)
  {
    dlg->getData(data);
    data->save(mConfig);
  }
  delete dlg;
}
void UserlevelDlg::slotDelete()
{
}

void UserlevelDlg::writeItems(KConfig *config)
{
  config->setGroup("Parameter");
  config->writeEntry("levellist",mList);
  config->sync();
}

void UserlevelDlg::setItems(const QStringList &list)
{
  for (QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
  {
    ListBox1->insertItem(*it);
    mList.append(*it);
  }

}



